Type: Package
Package: chevreulShiny
Title: Tools for managing SingleCellExperiment objects as projects
Version: 1.5.0
Authors@R: c(
    person("Kevin", "Stachelek", , "kevin.stachelek@gmail.com", 
    role = c("aut", "cre"), comment = c(ORCID = "0000-0003-2085-695X")),
    person("Bhavana", "Bhat", , "bhavanabhat29@gmail.com", role = "aut")
    )
Description: Tools for managing SingleCellExperiment objects as projects. 
    Includes functions for analysis and visualization of single-cell data. 
    Also included is a shiny app for visualization of pre-processed scRNA data.
    Supported by NIH grants R01CA137124 and R01EY026661 to David Cobrinik.
License: MIT + file LICENSE
URL: https://github.com/whtns/chevreulShiny,
        https://whtns.github.io/chevreulShiny/
Date: 2024-03-24
BugReports: https://github.com/cobriniklab/chevreulShiny/issues
Depends: R (>= 4.5.0), SingleCellExperiment, shiny (>= 1.6.0),
        shinydashboard, chevreulProcess, chevreulPlot
Imports: alabaster.base, clustree, ComplexHeatmap, DataEditR (>=
        0.0.9), DBI, dplyr, DT, EnhancedVolcano, fs, future, ggplot2,
        ggplotify, grDevices, methods, patchwork, plotly, purrr,
        rappdirs, readr, RSQLite, S4Vectors, scales, shinyFiles,
        shinyhelper, shinyjs, shinyWidgets, stats, stringr, tibble,
        tidyr, tidyselect, utils, waiter, wiggleplotr
Suggests: BiocStyle, knitr, RefManageR, rmarkdown, testthat (>= 3.0.0),
        EnsDb.Mmusculus.v79, EnsDb.Hsapiens.v86
VignetteBuilder: knitr
Encoding: UTF-8
LazyData: false
Roxygen: list(markdown = TRUE)
RoxygenNote: 7.3.1
biocViews: Coverage, RNASeq, Sequencing, Visualization, GeneExpression,
        Transcription, SingleCell, Transcriptomics, Normalization,
        Preprocessing, QualityControl, DimensionReduction, DataImport
Config/testthat/edition: 3
git_url: https://git.bioconductor.org/packages/chevreulShiny
git_branch: devel
git_last_commit: fa7a9d5
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 21:49:16 UTC; biocbuild
Author: Kevin Stachelek [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-2085-695X>),
  Bhavana Bhat [aut]
Maintainer: Kevin Stachelek <kevin.stachelek@gmail.com>

# Generated by roxygen2: do not edit by hand

export(list_plot_types)
export(load_bigwigs)
export(make_chevreul_clean_names)
export(minimalChevreulApp)
export(unite_metadata)
import(SingleCellExperiment)
import(chevreulPlot)
import(chevreulProcess)
import(ggplot2)
import(shiny)
importFrom(ComplexHeatmap,Heatmap)
importFrom(ComplexHeatmap,HeatmapAnnotation)
importFrom(ComplexHeatmap,draw)
importFrom(DBI,dbAppendTable)
importFrom(DBI,dbConnect)
importFrom(DBI,dbDisconnect)
importFrom(DBI,dbReadTable)
importFrom(DBI,dbWriteTable)
importFrom(DT,DTOutput)
importFrom(DT,datatable)
importFrom(DT,formatStyle)
importFrom(DT,renderDT)
importFrom(DataEditR,dataEditServer)
importFrom(DataEditR,dataEditUI)
importFrom(DataEditR,dataFilterServer)
importFrom(DataEditR,dataFilterUI)
importFrom(DataEditR,dataOutputServer)
importFrom(DataEditR,dataOutputUI)
importFrom(DataEditR,dataSelectServer)
importFrom(DataEditR,dataSelectUI)
importFrom(EnhancedVolcano,EnhancedVolcano)
importFrom(RSQLite,SQLite)
importFrom(S4Vectors,metadata)
importFrom(alabaster.base,readObject)
importFrom(alabaster.base,saveObject)
importFrom(clustree,clustree)
importFrom(dplyr,anti_join)
importFrom(dplyr,bind_cols)
importFrom(dplyr,bind_rows)
importFrom(dplyr,case_when)
importFrom(dplyr,distinct)
importFrom(dplyr,filter)
importFrom(dplyr,group_by)
importFrom(dplyr,left_join)
importFrom(dplyr,mutate)
importFrom(dplyr,pull)
importFrom(dplyr,select)
importFrom(dplyr,slice)
importFrom(dplyr,summarize)
importFrom(fs,dir_ls)
importFrom(fs,path)
importFrom(fs,path_dir)
importFrom(fs,path_file)
importFrom(future,plan)
importFrom(ggplotify,as.ggplot)
importFrom(grDevices,dev.off)
importFrom(grDevices,pdf)
importFrom(methods,is)
importFrom(patchwork,wrap_plots)
importFrom(plotly,config)
importFrom(plotly,event_data)
importFrom(plotly,ggplotly)
importFrom(plotly,layout)
importFrom(plotly,partial_bundle)
importFrom(plotly,plotlyOutput)
importFrom(plotly,renderPlotly)
importFrom(plotly,style)
importFrom(plotly,toWebGL)
importFrom(purrr,flatten_chr)
importFrom(purrr,map)
importFrom(purrr,map_chr)
importFrom(rappdirs,user_cache_dir)
importFrom(readr,read_csv)
importFrom(scales,hue_pal)
importFrom(scales,pseudo_log_trans)
importFrom(shiny,incProgress)
importFrom(shiny,selectInput)
importFrom(shiny,withProgress)
importFrom(shinyFiles,getVolumes)
importFrom(shinyFiles,parseFilePaths)
importFrom(shinyFiles,parseSavePath)
importFrom(shinyFiles,shinyFileChoose)
importFrom(shinyFiles,shinyFileSave)
importFrom(shinyFiles,shinyFilesButton)
importFrom(shinyFiles,shinySaveButton)
importFrom(shinyWidgets,dropdownButton)
importFrom(shinyWidgets,prettyRadioButtons)
importFrom(shinydashboard,box)
importFrom(shinydashboard,dashboardBody)
importFrom(shinydashboard,dashboardHeader)
importFrom(shinydashboard,dashboardSidebar)
importFrom(shinydashboard,menuItem)
importFrom(shinydashboard,sidebarMenu)
importFrom(shinydashboard,tabItem)
importFrom(shinydashboard,tabItems)
importFrom(shinyhelper,helper)
importFrom(shinyhelper,observe_helpers)
importFrom(shinyjs,alert)
importFrom(shinyjs,hidden)
importFrom(shinyjs,html)
importFrom(shinyjs,runcodeServer)
importFrom(shinyjs,runcodeUI)
importFrom(shinyjs,useShinyjs)
importFrom(stats,runif)
importFrom(stringr,str_count)
importFrom(stringr,str_detect)
importFrom(stringr,str_remove)
importFrom(stringr,str_replace)
importFrom(stringr,str_replace_all)
importFrom(stringr,str_subset)
importFrom(stringr,str_to_title)
importFrom(tibble,column_to_rownames)
importFrom(tibble,deframe)
importFrom(tibble,enframe)
importFrom(tibble,rownames_to_column)
importFrom(tibble,tibble)
importFrom(tidyr,drop_na)
importFrom(tidyr,pivot_wider)
importFrom(tidyr,unite)
importFrom(tidyselect,any_of)
importFrom(tidyselect,everything)
importFrom(utils,capture.output)
importFrom(utils,data)
importFrom(utils,head)
importFrom(utils,packageVersion)
importFrom(utils,sessionInfo)
importFrom(waiter,Waiter)
importFrom(waiter,spin_loaders)
importFrom(waiter,transparent)
importFrom(waiter,use_waiter)
importFrom(wiggleplotr,plotCoverageFromEnsembldb)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# chevreulShiny

This package includes a set of Shiny apps for exploring single cell RNA
datasets processed as a SingleCellExperiment

A demo using a human gene transcript dataset from Shayler et al. (link)
is available
<a href="http://cobrinik-1.saban-chla.usc.edu:8080/app/objectApp" target="_blank" rel="noopener noreferrer">here</a>

There are also convenient functions for:

- Clustering and Dimensional Reduction of Raw Sequencing Data.
- Integration and Label Transfer
- Louvain Clustering at a Range of Resolutions
- Cell cycle state regression and labeling

> \[!WARNING\] chevreulShiny was designed for full-length smart-seq
> based single cell data. Default settings may not be appropriate for
> droplet (10x) data, though most can be adjusted. Keep in mind best
> practices regarding normalization, dimensional reduction, etc. when
> using.

## Installation

You can install the released version of chevreulShiny from
<a href="https://github.com/whtns/chevreulShiny" target="_blank" rel="noopener noreferrer">github</a>
with:

### Install locally and run in three steps:

You can install chevreulShiny locally using the following steps:

``` r
install.packages("devtools")
devtools::install_github("whtns/chevreulShiny")
library(chevreulShiny)
create_project_db()
```

You can also customize the location of the app using these steps:

``` r
devtools::install_github("whtns/chevreul")
library(chevreulShiny)
create_project_db(destdir = "/your/path/to/app")
```

## TLDR

chevreulShiny provides a single command to:

- construct a SingleCellExperiment object

- filter genes by minimum expression and ubiquity

- normalize and scale expression by any of several methods packaged in
  SingleCellExperiment

## Run clustering on a single object

By default clustering will be run at ten different resolutions between
0.2 and 2.0. Any resolution can be specified by providing the resolution
argument as a numeric vector.

``` r
clustered_sce <- clustering_workflow(chevreul_sce,
    experiment_name = "sce_hu_trans",
    organism = "human"
)
```

## Get a first look at a processed dataset using an interactive shiny app

``` r
minimalChevreulApp(chevreul_sce)
```

#' Plot BigWig Coverage for Genes of Interest by a Given Variable
#'
#' Plot BigWig coverage for genes of interest colored by a given variable
#'
#' @param genes_of_interest Gene of interest
#' @param cell_metadata a dataframe with cell metadata from object
#' @param bigwig_tbl a tibble with colnames "name", "bigWig", and "sample_id"
#' matching the filename, absolute path, and sample name of each cell in the
#' cell_metadata
#' @param group_by Variable to color by
#' @param values_of_interest values of interest
#' @param organism human (default) or mouse
#' @param edb ensembldb object
#' @param heights The heights of each row in the grid of plot
#' @param scale_y whether to scale coverage
#' @param reverse_x whether to reverse x axis
#' @param start start coordinates
#' @param end end coordinates
#' @param summarize_transcripts whether to summarize transcript counts
#' @param ... extra arguments passed to plotCoverageFromEnsembldb
#'
#' @return a ggplot with coverage faceted by group_by
plot_gene_coverage_by_var <- function(
        genes_of_interest = "NRL",
        cell_metadata,
        bigwig_tbl,
        group_by = "batch",
        values_of_interest = NULL,
        organism = c("human", "mouse"),
        edb = NULL,
        heights = c(3, 1),
        scale_y = "log10",
        reverse_x = FALSE,
        start = NULL,
        end = NULL,
        summarize_transcripts = FALSE,
        ...) {
    organism <- match.arg(organism)
    if (missing(edb)) {
        
        if (!requireNamespace('EnsDb.Mmusculus.v79', quietly = TRUE))
            stop("Install 'EnsDb.Mmusculus.v79' to use this function.")
        
        if (!requireNamespace('EnsDb.Hsapiens.v86', quietly = TRUE))
            stop("Install 'EnsDb.Hsapiens.v86' to use this function.")
        
        edb <- switch(organism,
                      mouse = EnsDb.Mmusculus.v79::EnsDb.Mmusculus.v79,
                      human = EnsDb.Hsapiens.v86::EnsDb.Hsapiens.v86
        )
    }
    
    cell_metadata["sample_id"] <- NULL
    
    new_track_data <-
        cell_metadata |>
        rownames_to_column("sample_id") |>
        select(sample_id,
               condition = {{ group_by }},
               track_id = {{ group_by }},
               colour_group = {{ group_by }},
               everything()
        ) |>
        mutate(scaling_factor = 1) |> # rescale(nCount_RNA)
        mutate(condition = as.factor(condition),
               colour_group = as.factor(colour_group)) |>
        left_join(bigwig_tbl, by = "sample_id") |>
        filter(!is.na(bigWig)) |>
        identity()
    
    if (!is.null(values_of_interest)) {
        new_track_data <-
            new_track_data |>
            filter(condition %in% values_of_interest)
    }
    
    if (is.null(start) | is.null(end) | list(...)$rescale_introns == TRUE) {
        region_coords <- NULL
    } else {
        region_coords <- c(start, end)
    }
    
    coverage_plot_list <- plotCoverageFromEnsembldb(
        ensembldb = edb,
        gene_names = genes_of_interest,
        track_data = new_track_data,
        heights = heights,
        alpha = 0.5,
        fill_palette = hue_pal()(length(levels(new_track_data$colour_group))),
        return_subplots_list = TRUE,
        region_coords = region_coords,
        ...
    )
    
    if (scale_y == "log10") {
        coverage_plot_list$coverage_plot <-
            coverage_plot_list$coverage_plot +
            scale_y_continuous(trans = pseudo_log_trans(base = 10), 
                               breaks = 10^(0:4)) +
            NULL
    }
    
    if (reverse_x) {
        transformed_x_lim <- ggplot_build(
            coverage_plot_list$coverage_plot)$layout$panel_params[[1]]$x.range
        
        coverage_plot_list$coverage_plot <-
            coverage_plot_list$coverage_plot +
            coord_cartesian(
                xlim = rev(transformed_x_lim),
                expand = FALSE
            ) +
            NULL
        
        transformed_x_lim <- ggplot_build(
            coverage_plot_list$tx_structure)$layout$panel_params[[1]]$x.range
        coverage_plot_list$tx_structure <-
            coverage_plot_list$tx_structure +
            scale_x_reverse() +
            coord_cartesian(
                xlim = rev(transformed_x_lim),
                expand = FALSE
            ) +
            NULL
    }
    
    x_lim <- ggplot_build(
        coverage_plot_list$coverage_plot)$layout$panel_params[[1]]$x.range
    
    base_coverage <- coverage_plot_list$coverage_plot$data |>
        filter(!is.na(coverage)) |>
        group_by(sample_id, colour_group) |>
        summarize(sum = signif(sum(coverage), 4)) |>
        mutate(sum = sum / diff(x_lim)) |>
        identity()
    
    coverage_plot <- 
        wrap_plots(coverage_plot_list, heights = heights, ncol = 1)
    
    return(list(plot = coverage_plot, table = base_coverage))
}

#' Load Bigwigs
#'
#' Load a tibble of bigwig file paths by cell id
#'
#' @param object A object
#' @param bigwig_db Sqlite database of bigwig files
#'
#' @return a vector of bigwigs file paths
#' @export
load_bigwigs <- function(object, bigwig_db = "~/.cache/chevreul/bw-files.db") {
    con <- dbConnect(SQLite(), dbname = bigwig_db)
    
    bigwigfiles <- dbReadTable(con, "bigwigfiles") |>
        filter(sample_id %in% colnames(object)) |>
        identity()
    
    missing_bigwigs <- colnames(object)[!(colnames(object) %in% 
                                              bigwigfiles$sample_id)] |>
        paste(collapse = ", ")
    
    if(length(missing_bigwigs)>0){
        message(paste0("Sample coverage files ", missing_bigwigs, 
                       "(.bw) do not match samples in object (check file names)"))   
    }
    
    dbDisconnect(con)
    
    bigwigfiles <-
        bigwigfiles |>
        filter(sample_id %in% colnames(object))
    
    return(bigwigfiles)
}

#' @keywords internal
"_PACKAGE"

## usethis namespace: start
#' @importFrom alabaster.base readObject
#' @importFrom alabaster.base saveObject
#' @importFrom clustree clustree
#' @importFrom ComplexHeatmap draw
#' @importFrom ComplexHeatmap Heatmap
#' @importFrom ComplexHeatmap HeatmapAnnotation
#' @importFrom DataEditR dataEditServer
#' @importFrom DataEditR dataEditUI
#' @importFrom DataEditR dataFilterServer
#' @importFrom DataEditR dataFilterUI
#' @importFrom DataEditR dataOutputServer
#' @importFrom DataEditR dataOutputUI
#' @importFrom DataEditR dataSelectServer
#' @importFrom DataEditR dataSelectUI
#' @importFrom DBI dbAppendTable
#' @importFrom DBI dbConnect
#' @importFrom DBI dbDisconnect
#' @importFrom DBI dbReadTable
#' @importFrom DBI dbWriteTable
#' @importFrom dplyr anti_join
#' @importFrom dplyr bind_cols
#' @importFrom dplyr bind_rows
#' @importFrom dplyr case_when
#' @importFrom dplyr distinct
#' @importFrom dplyr filter
#' @importFrom dplyr group_by
#' @importFrom dplyr left_join
#' @importFrom dplyr mutate
#' @importFrom dplyr pull
#' @importFrom dplyr select
#' @importFrom dplyr slice
#' @importFrom dplyr summarize
#' @importFrom DT datatable
#' @importFrom DT DTOutput
#' @importFrom DT formatStyle
#' @importFrom DT renderDT
#' @importFrom EnhancedVolcano EnhancedVolcano
#' @importFrom fs dir_ls
#' @importFrom fs path
#' @importFrom fs path_dir
#' @importFrom fs path_file
#' @importFrom future plan
#' @importFrom ggplotify as.ggplot
#' @importFrom grDevices dev.off 
#' @importFrom grDevices pdf
#' @importFrom methods is
#' @importFrom patchwork wrap_plots
#' @importFrom plotly config
#' @importFrom plotly event_data
#' @importFrom plotly ggplotly
#' @importFrom plotly layout
#' @importFrom plotly partial_bundle
#' @importFrom plotly plotlyOutput
#' @importFrom plotly renderPlotly
#' @importFrom plotly style
#' @importFrom plotly toWebGL
#' @importFrom purrr flatten_chr
#' @importFrom purrr map
#' @importFrom purrr map_chr
#' @importFrom rappdirs user_cache_dir
#' @importFrom readr read_csv
#' @importFrom RSQLite SQLite
#' @importFrom S4Vectors metadata
#' @importFrom scales hue_pal
#' @importFrom scales pseudo_log_trans
#' @importFrom shiny incProgress
#' @importFrom shiny selectInput
#' @importFrom shiny withProgress
#' @importFrom shinydashboard box
#' @importFrom shinydashboard dashboardBody
#' @importFrom shinydashboard dashboardHeader
#' @importFrom shinydashboard dashboardSidebar
#' @importFrom shinydashboard menuItem
#' @importFrom shinydashboard sidebarMenu
#' @importFrom shinydashboard tabItem
#' @importFrom shinydashboard tabItems
#' @importFrom shinyFiles getVolumes
#' @importFrom shinyFiles parseFilePaths
#' @importFrom shinyFiles parseSavePath
#' @importFrom shinyFiles shinyFileChoose
#' @importFrom shinyFiles shinyFileSave
#' @importFrom shinyFiles shinyFilesButton
#' @importFrom shinyFiles shinySaveButton
#' @importFrom shinyhelper helper
#' @importFrom shinyhelper observe_helpers
#' @importFrom shinyjs alert
#' @importFrom shinyjs hidden
#' @importFrom shinyjs html
#' @importFrom shinyjs runcodeServer
#' @importFrom shinyjs runcodeUI
#' @importFrom shinyjs useShinyjs
#' @importFrom shinyWidgets prettyRadioButtons
#' @importFrom stats runif
#' @importFrom stringr str_count
#' @importFrom stringr str_detect
#' @importFrom stringr str_remove
#' @importFrom stringr str_replace
#' @importFrom stringr str_replace_all
#' @importFrom stringr str_subset
#' @importFrom stringr str_to_title
#' @importFrom tibble column_to_rownames
#' @importFrom tibble deframe
#' @importFrom tibble enframe
#' @importFrom tibble rownames_to_column
#' @importFrom tibble tibble
#' @importFrom tidyr drop_na
#' @importFrom tidyr pivot_wider
#' @importFrom tidyr unite
#' @importFrom tidyselect any_of
#' @importFrom tidyselect everything
#' @importFrom utils capture.output
#' @importFrom utils data
#' @importFrom utils head
#' @importFrom utils packageVersion
#' @importFrom utils sessionInfo
#' @importFrom waiter spin_loaders
#' @importFrom waiter transparent
#' @importFrom waiter use_waiter
#' @importFrom waiter Waiter
#' @importFrom wiggleplotr plotCoverageFromEnsembldb
## usethis namespace: end
#' @import chevreulProcess
#' @import chevreulPlot
#' @import ggplot2
#' @import SingleCellExperiment
#' @import shiny
NULL

plotClustree_UI <- function(id) {
    ns <- NS(id)
    tagList(
        chevreulBox(
            title = "Clustering Tree",
            plotOutput(ns("clustree")), height = "700px"
        )
    )
}

#' plot clustree server
#'
#' @noRd
plotClustree <- function(id, object) {
    moduleServer(id, function(input, output, session) {
        # set appropriate experiment
        experiment <- reactive({
            ifelse(query_experiment(object(), "integrated"), "integrated", "gene")
        })

        output$clustree <- renderPlot({
            req(object())
            experiment <- ifelse("integrated" %in% get_feature_types(object()),
                "integrated", "gene"
            )
            object <- set_feature_type(object(), experiment)
            clustree(object(), prefix = paste0(experiment, "_snn_res."))
        })
    })
}


#' plotViolin UI
#'
#' @param id
#'
#' @noRd
plotViolinui <- function(id) {
    ns <- NS(id)
    tagList(
        chevreulBox(
            title = "Violin Plots",
            uiOutput(ns("vln_group")),
            selectizeInput(ns("customFeature"),
                "Gene or transcript expression by which to color the plot
                eg. 'NRL' or 'ENST00000488147'",
                choices = NULL, multiple = FALSE
            ),
            radioButtons(ns("slot"), "Data Type",
                choices =
                    c(
                        "transformed" = "data",
                        "raw counts" = "counts"
                    )
            ),
            downloadButton(ns("downloadPlot")),
            plotlyOutput(ns("vplot"), height = 750),
            width = 11
        ) |>
            default_helper(
                type = "markdown", content = "violinPlot",
                size = "l"
            )
    )
}

#' Plot Violin Server
#'
#' Plots a Violin plot of a single data (gene expression, metrics, etc.)
#' in the server SingleCellExperiment app.
#'
#' @param object SingleCellExperiment object
#' @param featureType Gene or Transcript
#' @param organism_type Organism
#'
#' @noRd
plotViolin <- function(id, object, featureType, organism_type) {
    moduleServer(id, function(input, output, session) {
        ns <- session$ns

        prefill_feature <- reactive({
            req(featureType())
            switch(featureType(), transcript = "ENST00000488147", gene = "NRL")
        })
        observe({
            req(prefill_feature())
            req(object())
            updateSelectizeInput(session, "customFeature",
                choices = rownames(object()),
                selected = prefill_feature(), server = TRUE
            )
        })

        output$vln_group <- renderUI({
            req(object())
            selectizeInput(ns("vlnGroup"), "Grouping variable",
                choices = colnames(get_colData(object())), selected = "batch"
            )
        })

        vln_plot <- reactive({
            req(input$customFeature)
            req(input$vlnGroup)

            vln_plot <-
                plot_violin(object(),
                    group_by = input$vlnGroup,
                    features = input$customFeature, slot = input$slot
                )
        })

        output$downloadPlot <- downloadHandler(
            filename = function() {
                paste("violin", ".pdf", sep = "")
            },
            content = function(file) {
                ggsave(file, vln_plot(),
                width = 8, height = 6
                )
            }
        )

        output$vplot <- renderPlotly({
            req(object())
            req(input$vlnGroup)
            exclude_trace_number <-
                length(unique(get_colData(object())[[input$vlnGroup]])) * 2

            vln_plot <- ggplotly(vln_plot(), height = 700) |>
                style(opacity = 0.5) |>
                style(
                    hoverinfo = "skip",
                    traces = c(seq_len(exclude_trace_number))
                ) |>
                plotly_settings(width = 1200) |>
                # toWebGL() |>
                identity()
        })
    })
}


#' Plot Heatmap ui
#'
#' @param id
#'
#' @noRd
plotHeatmapui <- function(id) {
    ns <- NS(id)
    tagList(
        chevreulBox(
            title = "Heatmap",
            uiOutput(ns("colAnnoVarui")),
            radioButtons(ns("assayName"), "Data Scaling",
                choices = c(
                    scaled = "logcounts",
                    unscaled = "counts"
                ),
                selected = "logcounts", inline = TRUE
            ),
            selectizeInput(ns("dendroSelect"),
                "Clustering algorithm or metadata for column
                           arrangement",
                choices = NULL, selected = NULL,
                multiple = TRUE
            ),
            actionButton(ns("actionHeatmap"), "Plot Heatmap"),
            downloadButton(ns("downloadPlot"), "Download Heatmap"),
            selectizeInput(ns("customFeature"), "Gene or transcript
                           expression by which to color the plot; eg.
                           'NRL' or 'ENST00000488147'",
                choices = NULL, multiple = TRUE
            ),
            plotOutput(ns("heatmap"), height = 750),
            width = 12
        ) |>
            default_helper(type = "markdown", content = "heatMap")
    )
}

#' Plot Heatmap
#'
#' @param object a SingleCellExperiment object
#' @param featureType gene or transcript
#' @param organism_type human or mouse
#'
#' @noRd
#'
plotHeatmap <- function(id, object, featureType, organism_type) {
    stopifnot(is.reactive(object))


    moduleServer(id, function(input, output, session) {
        ns <- session$ns

        w <- Waiter$new("heatmap",
            html = spin_loaders(
                id = 1, color = "black",
                style = "position:relative;margin:auto;"
            ),
            color = transparent(.5)
        )

        observe({
            req(object())
            if (query_experiment(object(), "integrated")) {
                experiment <- "integrated"
            } else {
                experiment <- "gene"
            }

            preset_features <-
                get_variable_features(object(), experiment = experiment)[seq(50)]

            updateSelectizeInput(session, "customFeature",
                choices = get_features(object(), experiment = experiment),
                selected = preset_features, server = TRUE
            )
        })

        output$colAnnoVarui <- renderUI({
            req(object())

            formatted_col_names <- colnames(get_colData(object())) |>
                make_chevreul_clean_names()

            selectizeInput(ns("colAnnoVar"), "Column Annotation(s)",
                choices = formatted_col_names, selected = "batch", multiple = TRUE
            )
        })

        observe({
            req(object())

            hclust_methods <- c(
                "Ward" = "ward.D2", "single",
                "complete", "average"
            )

            updateSelectizeInput(
                session,
                "dendroSelect",
                choices = c(
                    hclust_methods,
                    colnames(get_colData(object()))
                ),
                selected = "ward.D2"
            )
        })

        heatmap_plot <- eventReactive(input$actionHeatmap, {
            req(input$customFeature)
            req(input$colAnnoVar)

            if (query_experiment(object(), "integrated")) {
                experiment <- "integrated"
            } else {
                experiment <- "gene"
            }

            hm <- make_complex_heatmap(object(),
                features = input$customFeature,
                experiment = experiment,
                group.by = input$colAnnoVar,
                assayName = input$assayName,
                col_arrangement = input$dendroSelect
            )
            return(hm)
        })

        output$heatmap <- renderPlot({
            req(heatmap_plot())
            w$show()
            draw(heatmap_plot())
        })

        output$downloadPlot <- downloadHandler(
            filename = function() {
                paste("heatmap", ".pdf", sep = "")
            },
            content = function(file) {
                ggsave(file, as.ggplot(heatmap_plot()) +
                    theme_void(),
                width = 8, height = 6
                )
            }
        )
    })
}

#' Integrate Project UI
#'
#'
#' @noRd
integrateProjui <- function(id) {
    ns <- NS(id)
    tagList(
        chevreulBox(
            title = "Integrate Projects",
            shinyFilesButton(ns("integratedProjDir"),
                label = "Integrated Project Directory",
                title = "Choose a directory to store integrated datasets", multiple = FALSE
            ),
            actionButton(ns("integrateAction"), "Integrate Selected Projects"),
            # useShinyjs(),
            # runcodeUI(code = "alert('Hello!')", id = "subsetcode"),
            textOutput(ns("integrationMessages")),
            checkboxInput(ns("legacySettings"), "Use Legacy Settings", value = FALSE),
            textOutput(ns("integrationResult")),
            shinySaveButton(ns("saveIntegratedProject"), "Save Integrated Project", "Save project as..."),
            DTOutput(ns("myDatatable")),
            width = 12
        ) |>
            default_helper(type = "markdown", content = "integrateProjects")
    )
}

#' Integrate Projects Server Function
#'
#' @param proj_matrices project matrices
#' @param object a SingleCellExperiment object
#' @param proj_dir project directory
#' @param con a connection
#'
#' @noRd
integrateProj <- function(id, proj_matrices, object, proj_dir, con) {
    moduleServer(id, function(input, output, session) {
        dataset_volumes <- reactive({
            dataset_volumes <- c(
                Home =
                    dirname(proj_dir()),
                "R Installation" = R.home(),
                getVolumes()()
            )
        })

        observe({
            req(dataset_volumes())
            shinyFileChoose(input, "integratedProjDir",
                roots = dataset_volumes(), session = session
            )
        })

        proj_matrix <- reactive({
            proj_matrices()$primary_projects
        })

        clean_proj_matrix <- reactive({
            subset(proj_matrix(), select = -c(project_path))
        })

        output$myDatatable <- renderDT(clean_proj_matrix(),
            server = FALSE,
            rownames = TRUE
        )

        selectedRows <- eventReactive(input$integrateAction, {
            input$myDatatable_rows_selected
        })

        selectedProjects <- reactive({
            selectedProjects <- slice(proj_matrix(), selectedRows()) |>
                pull(project_path) |>
                identity()
        })

        mergedObjects <- reactiveVal()

        observeEvent(input$integrateAction, {
            req(selectedProjects())
            withCallingHandlers(
                {
                    html("integrationMessages", "")
                    message("Beginning")
                    message(selectedProjects())
                    batches <- path(
                        selectedProjects(), "output",
                        "singlecellexperiment",
                        "unfiltered_sce.rds"
                    ) |>
                        map(readRDS)

                    names(batches) <- names(selectedProjects())
                    message(names(batches))
                    mergedObjects(
                        integration_workflow(
                            batches,
                            legacy_settings = input$legacySettings
                        )
                    )
                    # mergedObjects(batches[[1]])

                    message("Integration Complete!")
                },
                message = function(m) {
                    html(
                        id = "integrationMessages",
                        html = paste0("Running Integration: ", m$message),
                        add = FALSE
                    )
                }
            )
        })

        newProjDir <- reactive({
            req(mergedObjects())

            newProjName <- paste0(map(path_file(selectedProjects()), ~
                gsub("_proj", "", .x)), collapse = "_")
            newProjDir <- path(input$integratedProjDir, newProjName)

            proj_dir(newProjDir)

            newProjDir
        })


        volumes <- reactive({
            volumes <- c(
                Home = input$integratedProjDir,
                "R Installation" = R.home(),
                getVolumes()()
            )
            # print(volumes)
            volumes
        })

        observe({
            shinyFileSave(input,
                "saveIntegratedProject",
                roots = volumes(),
                session = session,
                restrictions = system.file(package = "base")
            )
        })


        integratedProjectSavePath <- eventReactive(input$saveIntegratedProject, {
            savefile <- parseSavePath(volumes(), input$saveIntegratedProject)

            savefile$datapath
        })

        output$integrationResult <- renderText({
            integratedProjectSavePath()
        })

        observeEvent(input$saveIntegratedProject, {
            req(mergedObjects())
            req(integratedProjectSavePath())

            if (!is.null(integratedProjectSavePath())) {
                withProgress(
                    message = paste0(
                        "Saving Integrated Dataset to ",
                        integratedProjectSavePath()
                    ),
                    value = 0,
                    {
                        # Sys.sleep(6)
                        incProgress(2 / 10)
                        save_sce(mergedObjects(),
                            proj_dir = integratedProjectSavePath()
                        )
                        writeLines(character(), path(
                            integratedProjectSavePath(),
                            ".here"
                        ))
                        # create_proj_db()
                        dbAppendTable(con, "projects_tbl", data.frame(
                            project_name = path_file(integratedProjectSavePath()),
                            project_path = integratedProjectSavePath(),
                            project_slug = str_remove(
                                path_file(integratedProjectSavePath()), "_proj$"
                            ),
                            project_type = "integrated_projects"
                        ))
                        incProgress(8 / 10)
                    }
                )
            }
        })


        return(integratedProjectSavePath)
    })
}


#' Change Embedding Parameters UI
#'
#' @noRd
changeEmbedParamsui <- function(id) {
    ns <- NS(id)
    minDist_vals <- prep_slider_values(0.3)
    negsamprate_vals <- prep_slider_values(5)

    tagList(
        selectizeInput(ns("dims"),
            label = "Dimensions from PCA",
            choices = seq(1, 99), multiple = TRUE,
            selected = seq(30)
        ),
        sliderInput(ns("minDist"),
            label = "Minimum Distance",
            min = minDist_vals$min, max = minDist_vals$max,
            value = minDist_vals$value, step = minDist_vals$step
        ),
        sliderInput(ns("negativeSampleRate"),
            label = "Negative Sample Rate",
            min = negsamprate_vals$min, max = negsamprate_vals$max,
            value = negsamprate_vals$value,
            step = negsamprate_vals$step
        )
    )
}

#' Change Embedding Parameters
#'
#' @param object a SingleCellExperiment object
#'
#' @noRd
changeEmbedParams <- function(id, object) {
    moduleServer(id, function(input, output, session) {
        object <- RunUMAP(object(),
            dims = as.numeric(input$dims),
            reduction = "PCA", min.dist = input$minDist,
            negative.sample.rate = input$negativeSampleRate
        )

        return(object)
    })
}

#' Plot Dimensional Reduduction UI
#'
#' @noRd
plotDimRedui <- function(id) {
    ns <- NS(id)
    chevreulBox(
        title = "Embedding",
        chevreulDropDownButton(
            ns("dimPlotSettings"),
            selectizeInput(ns("embedding"), "Embedding",
                choices = NULL,
                selected = NULL
            ),
            sliderInput(ns("dotSize"), "Size of Points in UMAP",
                min = 0.5,
                max = 2, step = 0.1, value = 1
            ),
            selectizeInput(ns("dim1"), "Dimension 1",
                choices = seq(1, 99),
                selected = 1
            ),
            selectizeInput(ns("dim2"), "Dimension 2",
                choices = seq(1, 99),
                selected = 2
            )
        ),
        selectizeInput(ns("plottype"), "Variable to Plot",
            choices = NULL,
            multiple = TRUE
        ),
        selectizeInput(ns("customFeature"),
            "Gene or transcript expression by which to color
                       the plot; eg. 'NRL' or 'ENST00000488147'",
            choices = NULL, multiple = FALSE
        ),
        plotlyOutput(ns("dplot"), height = 500),
        width = 6
    )
}

#' Plot Dimensional Reduduction
#'
#' @param object a SingleCellExperiment object
#' @param plot_types plot types
#' @param featureType gene or transcript
#' @param organism_type human or mouse
#' @param reductions embeddings
#'
#' @noRd

plotDimRed <- function(id, object, plot_types, featureType,
                       organism_type, reductions) {
    moduleServer(id, function(input, output, session) {
        ns <- session$ns
        output$myPanel <- renderUI({
            req()
            lev <- sort(unique(input$select))
            cols <- gg_fill_hue(length(lev))

            # New IDs "colX1" so that it partly coincide with input$select...
            lapply(seq_along(lev), function(i) {
                colourInput(
                    inputId = paste0("col", lev[i]),
                    label = paste0("Choose colour for ", lev[i]),
                    value = cols[i]
                )
            })
        })

        observe({
            req(object())
            updateSelectizeInput(session, "embedding",
                choices = reductions(),
                selected = rev(reductions())[1], server = TRUE
            )
        })

        selected_plot <- reactiveVal()
        observe({
            req(object())
            # selected_plot <- ifelse(is.null(selected_plot()), "louvain",
            #                         selected_plot())
            updateSelectizeInput(session, "plottype",
                choices = flatten_chr(plot_types()),
                selected = "batch"
            )
        })
        prefill_feature <- reactive({
            req(featureType())
            switch(featureType(), transcript = "ENST00000488147", gene = "NRL")
        })
        observe({
            req(prefill_feature())
            req(featureType())
            req(object())
            updateSelectizeInput(session, "customFeature",
                choices = get_features(object(), featureType()),
                selected = prefill_feature(), server = TRUE
            )
        })

        output$dplot <- renderPlotly({
            req(input$plottype)
            req(object())
            req(input$embedding)
            if (length(input$plottype) > 1) {
                cross_plot_sce <- unite_metadata(object(), input$plottype)

                newcolname <- paste(input$plottype, collapse = "_")
                cross_plot_sce[[newcolname]] <- Idents(cross_plot_sce)

                selected_plot(newcolname)

                plot_colData_on_embedding(cross_plot_sce,
                    dims = c(input$dim1, input$dim2),
                    embedding = input$embedding, group = NULL,
                    point_size = input$dotSize,
                    return_plotly = TRUE
                )
            } else {
                if (input$plottype == "feature") {
                    plot_feature_on_embedding(object(),
                        dims = c(
                            input$dim1,
                            input$dim2
                        ), embedding = input$embedding,
                        features = input$customFeature, point_size = input$dotSize,
                        return_plotly = TRUE
                    )
                } else if (input$plottype %in% plot_types()$continuous_vars) {
                    plot_feature_on_embedding(object(),
                        dims = c(
                            input$dim1,
                            input$dim2
                        ), embedding = input$embedding,
                        features = input$plottype, point_size = input$dotSize,
                        return_plotly = TRUE
                    )
                } else if (input$plottype %in% plot_types()$category_vars) {
                    plot_colData_on_embedding(object(),
                        dims = c(input$dim1, input$dim2),
                        embedding = input$embedding, group = input$plottype,
                        point_size = input$dotSize,
                        return_plotly = TRUE
                    )
                }
            }
        })
    })
}


#' Create Table of Selected Cells UI
#'
#' @param id
#'
#' @noRd
tableSelectedui <- function(id) {
    ns <- NS(id)

    tagList(DTOutput(ns("brushtable")))
}

#' Create Table of Selected Cells
#'
#' @param object a SingleCellExperiment object
#'
#' @noRd
tableSelected <- function(id, object) {
    moduleServer(id, function(input, output, session) {
        brush <- reactive({
            req(object())
            d <- event_data("plotly_selected")
            if (is.null(d)) {
                "Click and drag events (i.e. select/lasso) 
                appear here (double-click to clear)"
                return(d)
            } else {
                # selected_cells <- colnames(object())[as.numeric(d$key)]
                d$key
            }
        })

        output$brushtable <- renderDT({
            req(object())
            req(brush())
            selected_meta <- data.frame(get_colData(object())[brush(), ])
            datatable(selected_meta,
                extensions = "Buttons",
                selection = list(
                    mode = "multiple",
                    selected = seq_len(nrow(selected_meta)),
                    target = "row"
                ),
                options = list(
                    dom = "Bft", buttons = c("copy", "csv"),
                    scrollX = "100px", scrollY = "800px"
                )
            )
        })

        selected_cells <- reactive({
            selected_rows <- input$brushtable_rows_selected
            rownames(get_colData(object())[brush(), ])[selected_rows]
        })

        return(selected_cells)
    })
}


#' Differential Expression UI
#'
#' @noRd
diffexui <- function(id) {
    ns <- NS(id)
    tagList(
        chevreulBox(
            title = "Differential Expression Settings",
            radioButtons(ns("diffex_scheme"),
                "Cells to Compare",
                choiceNames =
                    c("SingleCellExperiment Cluster", "Custom Selection"),
                choiceValues = c("louvain", "custom"),
                selected = "louvain",
                inline = TRUE
            ),
            conditionalPanel(
                ns = ns,
                condition = "input.diffex_scheme == 'louvain'",
                sliderInput(ns("objectResolution"),
                    "Resolution of clustering algorithm (affects number of clusters)",
                    min = 0.2, max = 2, step = 0.2, value = 0.6
                ),
                numericInput(ns("cluster1"),
                    "first cluster to compare",
                    value = 0
                ), numericInput(ns("cluster2"),
                    "second cluster to compare",
                    value = 1
                )
            ),
            conditionalPanel(
                ns = ns,
                condition = "input.diffex_scheme == 'custom'",
                sliderInput(ns("customResolution"), "Resolution of clustering
                            algorithm (affects number of clusters)",
                    min = 0.2, max = 2, step = 0.2, value = 0.6
                ),
                actionButton(
                    ns("saveClust1"),
                    "Save to Custom Cluster 1"
                ), actionButton(
                    ns("saveClust2"),
                    "Save to Custom Cluster 2"
                )
            ),
            uiOutput(ns("testChoices")),
            radioButtons(ns("featureType"), "Features to Compare",
                choices = c("gene", "transcript")
            ),
            actionButton(
                ns("diffex"),
                "Run Differential Expression"
            ),
            downloadLink(ns("downloadData"), "Download Complete DE Results"),
            DTOutput(ns("DT1")),
            width = 6
        ),
        chevreulBox(
            title = "Volcano Plot",
            sliderInput(ns("FCcutoff"), "FC cutoff value (log2 fold change)",
                min = 0, max = 10, step = 0.5, value = 1
            ),
            sliderInput(ns("pValCutoff"), "-log10 p adj value",
                min = 0, max = 5, step = 0.5, value = 1.5
            ),
            plotOutput(ns("volcano")),
            downloadButton(ns("downloadVolcanoPlot"), "Download Volcano Plot"),
            width = 6
        ),
        # chevreulBox(
        #   title = "Cells",
        #   tabsetPanel(type = "tabs",
        #               tabPanel("Selected Cells", tableSelectedui("diffex")),
        #               tabPanel("Custom Cluster 1", DTOutput("cc1")),
        #               tabPanel("Custom Cluster 2", DTOutput("cc2"))        #   ),
        #   width = 6
        # ),

        chevreulBox(
            title = "Selected Cells",
            tableSelectedui("diffex"),
            width = 12
        ),
        chevreulBox(
            title = "Custom Cluster 1", DTOutput(ns("cc1")),
            width = 6
        ), chevreulBox(
            title = "Custom Cluster 2", DTOutput(ns("cc2")),
            width = 6
        ),
    )
}


#' Title
#'
#' @noRd
cells_selected <- function(input) {
    if (identical(input, character(0))) {
        "Please selected desired cells by clicking on the table"
    } else {
        NULL
    }
}

#' Differential Expression
#'
#' @param object a SingleCellExperiment object
#' @param featureType gene or transcript
#' @param selected_cells selected cells
#' @param tests tests to use
#'
#' @noRd
diffex <- function(id, object, featureType, selected_cells,
                   tests = c(
                       "t-test" = "t", "wilcoxon rank-sum test" = "wilcox",
                       "Likelihood-ratio test (bimodal)" = "bimod",
                       "MAST" = "MAST"
                   )) {
    moduleServer(id, function(input, output, session) {
        ns <- session$ns

        experiment <- reactive({
            req(object())
            if (query_experiment(object(), "integrated")) {
                experiment <- "integrated"
            } else {
                experiment <- "gene"
            }
        })

        output$testChoices <- renderUI(
            selectizeInput(ns("diffex_method"),
                "Method of Differential Expression",
                choices = tests,
                selected = "t"
            )
        )

        custom_cluster1 <- eventReactive(input$saveClust1, {
            validate(
                cells_selected(selected_cells())
            )
            isolate(selected_cells())
        })
        custom_cluster2 <- eventReactive(input$saveClust2, {
            validate(
                cells_selected(selected_cells())
            )
            isolate(selected_cells())
        })

        output$cc1 <- renderDT({
            req(custom_cluster1())
            selected_meta <-
                data.frame(get_colData(object())[custom_cluster1(), ])
            datatable(selected_meta,
                extensions = "Buttons",
                options = list(dom = "Bft", buttons = c(
                    "copy",
                    "csv"
                ), scrollX = "100px", scrollY = "400px")
            )
        })
        output$cc2 <- renderDT({
            req(custom_cluster2())
            selected_meta <-
                data.frame(get_colData(object())[custom_cluster2(), ])
            datatable(selected_meta,
                extensions = "Buttons",
                options = list(dom = "Bft", buttons = c(
                    "copy",
                    "csv"
                ), scrollX = "100px", scrollY = "400px")
            )
        })

        de_results <- eventReactive(input$diffex, {
            if (input$diffex_scheme == "louvain") {
                sce_de(object(), input$cluster1, input$cluster2,
                    resolution = input$objectResolution,
                    diffex_scheme = input$diffex_scheme,
                    input$featureType, tests = input$diffex_method
                )
            } else if (input$diffex_scheme == "custom") {
                # req(custom_cluster1())
                # req(custom_cluster2())
                cluster1 <- unlist(strsplit(
                    custom_cluster1(),
                    " "
                ))
                cluster2 <- unlist(strsplit(
                    custom_cluster2(),
                    " "
                ))
                sce_de(object(), cluster1, cluster2,
                    input$customResolution,
                    diffex_scheme = input$diffex_scheme, input$featureType,
                    tests = input$diffex_method
                )
            }
        })

        output$DT1 <- renderDT(de_results()[[input$diffex_method]],
            extensions = "Buttons", options = list(
                dom = "Bfptr",
                buttons = c("copy", "csv"), scrollX = "100px", scrollY = "600px"
            ), class = "display"
        )

        Volcano <- reactive({
            de_input <- de_results()[[input$diffex_method]] |>
                distinct(symbol, .keep_all = TRUE) |>
                column_to_rownames("symbol")

            EnhancedVolcano(
                de_input,
                lab = rownames(de_input),
                x = "avg_log2FC",
                y = "p_val_adj",
                pCutoff = 1 / (10^as.numeric(input$pValCutoff)),
                FCcutoff = as.numeric(input$FCcutoff),
                raster = TRUE
            )
        })

        output$volcano <- renderPlot({
            print(Volcano())
        })

        output$downloadVolcanoPlot <- downloadHandler(
            filename = function() {
                paste("DE_Volcano_plot", ".pdf", sep = "")
            },
            content = function(file) {
                ggsave(file, Volcano(),
                width = 8, height = 6
                )
            }
        )

        cluster_list <- reactive({
            if (input$diffex_scheme == "louvain") {
                sce_meta <-
                    get_colData(object())[[paste0(
                        DefaultAssay(object()),
                        "_snn_res.",
                        input$objectResolution
                    )]]
                cluster1_cells <- rownames(
                    sce_meta[sce_meta == input$cluster1, , drop = FALSE]
                )
                cluster2_cells <- rownames(
                    sce_meta[sce_meta == input$cluster2, , drop = FALSE]
                )
                list(cluster1 = cluster1_cells, cluster2 = cluster2_cells)
            } else if (input$diffex_scheme == "feature") {
                list(cluster1 = custom_cluster1(), cluster2 = custom_cluster2())
            }
        })

        return(list(cluster_list = cluster_list, de_results = de_results))
    })
}


#' Find Markers UI
#'
#' @noRd
chevreulMarkersui <- function(id) {
    ns <- NS(id)
    tagList(
        chevreulBox(
            title = "Find Markers",
            uiOutput(ns("dplottype")),
            sliderInput(ns("resolution2"),
                label = "Resolution of clustering algorithm (affects number of clusters)",
                min = 0.2, max = 2, step = 0.2, value = 0.6
            ),
            numericInput(ns("num_markers"),
                "Select Number of Markers to Plot for Each Value",
                value = 5, min = 2, max = 20
            ),
            uiOutput(ns("valueSelect")),
            radioButtons(ns("markerMethod"), "Method of Marker Selection",
                choices = c("wilcox"), selected = "wilcox",
                inline = TRUE
            ),
            sliderInput(ns("pValCutoff"), "P Value cutoff",
                min = 0.01,
                max = 1, value = 1
            ),
            selectizeInput(ns("dotFeature"), "Feature for Marker Plot",
                choices = NULL
            ),
            actionButton(ns("plotDots"), "Plot Markers!"),
            downloadButton(ns("downloadMarkerTable"), "Download Markers!"),
            checkboxInput(ns("uniqueMarkers"), "Make Markers Unique",
                value = FALSE
            ),
            checkboxInput(ns("hidePseudo"), "Hide Pseudogenes", value = TRUE),
            plotlyOutput(ns("markerplot"), height = 800),
            width = 6
        )
    ) |>
        default_helper(type = "markdown", content = "findMarkers")
}

#' Find Markers
#'
#' @param object a SingleCellExperiment object
#'
#' @noRd
chevreulMarkers <- function(id, object, plot_types, featureType) {
    moduleServer(id, function(input, output, session) {
        ns <- session$ns

        observe({
            req(object())
            updateSelectizeInput(session, "dotFeature",
                choices = c(
                    mainExpName(object()),
                    altExpNames(object())
                ),
                selected = "gene", server = TRUE
            )
        })

        output$dplottype <- renderUI({
            req(object())
            # selected_plot <- ifelse(is.null(selected_plot()), "louvain",
            #                         selected_plot())
            selectizeInput(ns("plottype"), "Variable to Plot",
                choices = flatten_chr(plot_types()),
                selected = "louvain", multiple = TRUE
            )
        })

        experiment <- reactive({
            req(object())
            if (query_experiment(object(), "integrated")) {
                experiment <- "integrated"
            } else {
                experiment <- "gene"
            }
        })

        group_by <- reactive({
            req(input$plottype)

            if (input$plottype == "louvain") {
                group_by <- paste0(experiment(), "_snn_res.", input$resolution2)
            } else {
                group_by <- input$plottype
            }
        })

        output$valueSelect <- renderUI({
            req(object())
            req(group_by())

            choices <- levels(factor(get_colData(object())[[group_by()]]))

            selectizeInput(ns("displayValues"), "Values to display",
                multiple = TRUE, choices = choices
            )
        })

        marker_plot_return <- eventReactive(input$plotDots, {
            plot_marker_features(object(),
                group_by = group_by(),
                num_markers = input$num_markers,
                selected_values = input$displayValues,
                marker_method = input$markerMethod,
                experiment = input$dotFeature,
                featureType = featureType(),
                hide_pseudo = input$hidePseudo,
                unique_markers = input$uniqueMarkers,
                p_val_cutoff = input$pValCutoff, return_plotly = TRUE
            )
        })

        output$markerplot <- renderPlotly({
            # req(input$displayClusters)
            marker_plot_return()$plot
        })

        output$downloadMarkerTable <- downloadHandler(
            filename = function() {
                paste(group_by(), "_markers.csv", sep = "")
            },
            content = function(file) {
                write_csv(marker_plot_return()$markers, file)
            }
        )
    })
}

#' Plot Read Count UI
#'
#' @noRd
plotReadCountui <- function(id) {
    ns <- NS(id)
    chevreulBox(
        title = "Histogram (Read Counts, etc.)",
        uiOutput(ns("group_byui")),
        uiOutput(ns("colorbyui")),
        sliderInput(ns("resolution"),
            "Resolution of clustering algorithm
                    (affects number of clusters)",
            min = 0.2, max = 2, step = 0.2, value = 0.6
        ),
        radioButtons(ns("yScale"), "Y axis transforamtion",
            choices = c("log", "linear"), selected = "linear",
            inline = TRUE
        ),
        plotlyOutput(ns("rcplot"), height = 500),
        collapsed = TRUE
    )
}

#' Plot Read Count
#'
#' @param object a SingleCellExperiment object
#' @param plot_types plot types
#'
#' @noRd
plotReadCount <- function(id, object, plot_types) {
    moduleServer(id, function(input, output, session) {
        ns <- session$ns

        output$colorbyui <- renderUI({
            req(object())
            selectInput(ns("colorby"), "Variable to Color the Plot by",
                choices = flatten_chr(plot_types()), selected = c("louvain"),
                multiple = FALSE
            )
        })

        output$group_byui <- renderUI({
            req(object())
            selectInput(ns("group_by"), "Variable for x-axis",
                choices = flatten_chr(plot_types()), selected = c("nCount_RNA"),
                multiple = FALSE
            )
        })

        output$rcplot <- renderPlotly({
            req(object())
            req(input$colorby)

            if (input$colorby == "louvain") {
                if (query_experiment(object(), "integrated")) {
                    experiment <- "integrated"
                } else {
                    experiment <- "gene"
                }

                louvain_resolution <- paste0(
                    experiment, "_snn_res.",
                    input$resolution
                )
                plot_colData_histogram(object(),
                    group_by = input$group_by,
                    fill_by = louvain_resolution,
                    yscale = input$yScale, return_plotly = TRUE
                )
            } else if (input$colorby %in% flatten_chr(plot_types())) {
                plot_colData_histogram(object(),
                    group_by = input$group_by,
                    fill_by = input$colorby, yscale = input$yScale,
                    return_plotly = TRUE
                )
            }
        })
    })
}

#' Cell Cycle Score UI
#'
#' @noRd
ccScoreui <- function(id) {
    tagList()
}

#' Cell Cycle Score
#'
#'
#' @noRd
ccScore <- function(id) {
    moduleServer(id, function(input, output, session) {
        output$rplot1 <- renderPlot({
            req(object())
            plot_ridge(object(), features = input$feature)
        })
        plotOutput("rplot1", height = 750)
    })
}

#' Plot All Transcripts UI Module
#'
#' @noRd
allTranscriptsui <- function(id) {
    ns <- NS(id)
    tagList(
        default_helper(
            chevreulBox(
                title = "Transcript Expression per Gene",
                selectizeInput(ns("embeddingGene"), "Gene or transcript expression by which to color the plot; eg. 'NRL'", choices = NULL, selected = NULL),
                selectizeInput(ns("transcriptSelect"), "Transcript to Plot", choices = NULL),
                downloadButton(ns("downloadPlot"), "Download Transcript Plots"),
                selectizeInput(ns("embedding"), "Embedding", choices = NULL, selected = NULL),
                plotlyOutput(ns("transcriptPlot")),
                # uiOutput("plotlys"),
                width = 6
            ),
            type = "markdown", content = "allTranscripts"
        ),
        default_helper(
            chevreulBox(
                title = "Transcript Expression per Gene",
                selectizeInput(ns("compositionGene"), "Gene or transcript expression by which to color the plot; eg. 'NRL'", choices = NULL, selected = NULL),
                selectizeInput(ns("groupby"), "Group by:", choices = NULL, selected = NULL),
                actionButton(ns("plotComposition"), "Plot transcript composition"),
                checkboxInput(ns("standardizeExpression"), "Standardize Expression", value = FALSE),
                checkboxInput(ns("dropZero"), "Drop Zero Values", value = FALSE),
                plotlyOutput(ns("compositionPlot")),
                DTOutput(ns("compositionDT")),
                width = 6
            ),
            type = "markdown", content = "allTranscripts"
        )
    )
}

#' Plot All Transcripts Server
#'
#' @param object a SingleCellExperiment object
#' @param featureType gene or transcript
#'
#' @noRd
allTranscripts <- function(id, object, featureType, organism_type) {
    moduleServer(id, function(input, output, session) {
        observe({
            req(object())
            updateSelectizeInput(session, "compositionGene",
                choices = get_features(object(),
                    experiment = "gene"
                ),
                selected = "NRL", server = TRUE
            )
            updateSelectizeInput(session, "embeddingGene",
                choices = get_features(object(),
                    experiment = "gene"
                ),
                selected = "NRL", server = TRUE
            )

            formatted_col_names <- colnames(get_colData(object())) |>
                make_chevreul_clean_names()

            updateSelectizeInput(session, "groupby",
                choices = formatted_col_names,
                selected = "batch", server = TRUE
            )
        })

        transcripts <- reactive({
            req(object())
            req(input$embeddingGene)
            if (query_experiment(object(), "transcript")) {
                get_transcripts_from_sce(object(), input$embeddingGene
                )
            }
        })

        observe({
            req(object())
            req(transcripts())
            updateSelectizeInput(session, "embedding", choices = c("PCA", "TSNE", "UMAP"), selected = "UMAP", server = TRUE)
            updateSelectizeInput(session, "transcriptSelect", choices = transcripts(), server = TRUE)
        })

        composition_plot <- eventReactive(input$plotComposition, {
            plot_transcript_composition(object(),
                gene_symbol = input$compositionGene,
                group.by = input$groupby,
                standardize = input$standardizeExpression,
                drop_zero = input$dropZero
            )
        })

        output$compositionPlot <- renderPlotly({
            composition_plot()$plot |>
                ggplotly(height = 400) |>
                plotly_settings() |>
                toWebGL() |>
                # partial_bundle() |>
                identity()
        })

        output$compositionDT <- renderDT({
            datatable(composition_plot()$data,
                extensions = "Buttons",
                options = list(dom = "Bft", buttons = c(
                    "copy",
                    "csv"
                ), scrollX = "100px", scrollY = "400px")
            )
        })

        pList <- reactive({
            req(transcripts())
            req(input$embedding)
            pList <- plot_all_transcripts(object(), transcripts(), input$embedding,
                from_gene = FALSE, point_size = 1
            )
        })

        output$transcriptPlot <- renderPlotly({
            req(pList())
            pList()[[input$transcriptSelect]] |>
                ggplotly(height = 400) |>
                plotly_settings() |>
                toWebGL()
        })

        output$downloadPlot <- downloadHandler(
            filename = function() {
                paste(input$embeddingGene, "_transcripts.pdf", sep = "")
            },
            content = function(file) {
                pdf(file, width = 5, height = 4)
                map(pList(), print)
                dev.off()
            }
        )
    })
}

#' Title
#'
#' @noRd
techInfoui <- function(id) {
    ns <- NS(id)

    fluidRow(
        chevreulBox(
            title = "Information about samples and analysis",
            htmlOutput(ns("sample_info_general")),
            width = 12
        )
    )
}

#' Title
#'
#' @param object a SingleCellExperiment object
#'
#' @noRd
techInfo <- function(id, object) {
    moduleServer(id, function(input, output, session) {
        sce_metadata <- reactive({
            req(object())
            metadata(object())$experiment
        })

        observe({
            # general info
            output$sample_info_general <- renderText({
                info <- paste0(
                    "<strong><u>General</u></strong>",
                    "<ul>",
                    "<li><b>Date of analysis:</b> ",
                    sce_metadata()$date_of_analysis,
                    "<li><b>Date of export:</b> ",
                    sce_metadata()$date_of_export,
                    "<li><b>Experiment name:</b> ",
                    sce_metadata()$experiment_name,
                    "<li><b>Organism:</b> ",
                    sce_metadata()$organism,
                    "</ul>",
                    "<strong><u>Parameters</u></strong>",
                    "<ul>",
                    "<li><b>Discard genes in fewer than X cells:</b> ",
                    sce_metadata()$parameters$discard_genes_expressed_in_fewer_cells_than,
                    "<li><b>Keep mitochondrial genes:</b> ",
                    sce_metadata()$parameters$keep_mitochondrial_genes,
                    "<li><b>Min/max # of UMI:</b> ",
                    paste0(
                        sce_metadata()$filtering$UMI_min, " / ",
                        sce_metadata()$filtering$UMI_max
                    ),
                    "<li><b>Min/max # of expressed genes:</b> ",
                    paste0(
                        sce_metadata()$filtering$genes_min, " / ",
                        sce_metadata()$filtering$genes_max
                    ),
                    "<li><b>Cluster resolution: </b>",
                    paste(sce_metadata()$parameters$cluster_resolution, collapse = ","),
                    "<li><b>Number of principal components: </b>",
                    sce_metadata()$parameters$number_PCs,
                    "<li><b>Variables to regress: </b>",
                    sce_metadata()$parameters$variables_to_regress_out,
                    "<li><b>tSNE perplexity: </b>",
                    sce_metadata()$parameters$tSNE_perplexity,
                    "</ul>",
                    "<strong><u>Marker genes</u></strong>",
                    "<ul>",
                    "<li><b>p-value threshold:</b> ",
                    "0.05",
                    "</ul>"
                )
                paste0(
                    info,
                    "<strong><u>Technical info (package versions)</u></strong>",
                    "<ul>",
                    "<li><strong>chevreulShiny version:</strong> ",
                    sce_metadata()$chevreul_version,
                    "<li><strong>SingleCellExperiment version:</strong> ",
                    sce_metadata()$SingleCellExperiment_version,
                    "<li><strong>Session info:</strong> ",
                    "</ul>",
                    "<pre>",
                    sce_metadata()$sessionInfo,
                    "</pre>"
                )
            })
        })
    })
}


#' Title
#'
#' @noRd
plotCoverage_UI <- function(id) {
    ns <- NS(id)

    tagList(
        chevreulBox(
            title = "Plot Coverage",
            selectizeInput(ns("geneSelect"), "Select a Gene",
                choices = NULL,
                selected = "NRL", multiple = FALSE
            ),
            selectizeInput(ns("varSelect"), "Color by Variable",
                choices = NULL, multiple = FALSE
            ),
            actionButton(ns("plotCoverage"), "Plot Coverage"),
            downloadButton(ns("downloadPlot"), "Download Coverage Plot"),
            uiOutput(ns("displayvaluesui")),
            br(),
            chevreulDropDownButton(ns("coveragePlotSettings"),
            checkboxInput(ns("collapseIntrons"), "Collapse Introns",
                value = TRUE
            ),
            checkboxInput(ns("meanCoverage"),
                "Summarize Coverage to Mean",
                value = TRUE
            ),
            checkboxInput(ns("summarizeTranscripts"),
                "Summarize transcript models to gene",
                value = FALSE
            ),
            radioButtons(ns("yScale"), "Scale Y Axis",
                choices = c("absolute", "log10"),
                selected = "log10"
            ),
            numericInput(ns("start"), "start coordinate", value = NULL),
            numericInput(ns("end"), "end coordinate", value = NULL)),
            DTOutput(ns("coverageTable")),
            plotOutput(ns("coveragePlot"), height = "1500px"),
            width = 12
        )
    )
}


#' Plot Coverage Module
#'
#' @param object a SingleCellExperiment object
#' @param plot_types plot types
#' @param bigwig_dir bigwig directory
#' @param organism_type human or mouse
#'
#' @noRd
plotCoverage <- function(id, object, plot_types, proj_dir,
                         organism_type = "human",
                         bigwig_db = "~/.cache/chevreul/bw-files.db") {
    moduleServer(id, function(input, output, session) {
        ns <- session$ns

        w <- Waiter$new("coveragePlot",
            html = spin_loaders(
                id = 1, color = "black",
                style = "position:relative;margin:auto;"
            ),
            color = transparent(.5)
        )

        observe({
            req(object())
            updateSelectizeInput(session, "geneSelect",
                choices = get_features(object(),
                    experiment = "gene"
                ),
                server = TRUE
            )

            formatted_col_names <- colnames(get_colData(object())) |>
                make_chevreul_clean_names()

            updateSelectizeInput(session, "varSelect",
                choices = formatted_col_names, selected = "batch"
            )
        })

        displayvalues <- reactive({
            req(input$varSelect)
            req(object())
            unique(object()[][[input$varSelect]])
        })

        output$displayvaluesui <- renderUI({
            req(input$varSelect)
            selectizeInput(ns("displayvalues"), "groups to display",
                choices = displayvalues(), multiple = TRUE
            )
        })

        bigwig_tbl <- reactive({
            load_bigwigs(object(), bigwig_db)
        })

        coverage_return <- eventReactive(input$plotCoverage, {
            req(object())
            req(bigwig_tbl())

            plot_gene_coverage_by_var(
                genes_of_interest = input$geneSelect,
                cell_metadata = get_colData(object()),
                bigwig_tbl = bigwig_tbl(),
                group_by = input$varSelect,
                values_of_interest = input$displayvalues,
                organism = metadata(object())$experiment$organism,
                mean_only = input$meanCoverage,
                rescale_introns = input$collapseIntrons,
                scale_y = input$yScale,
                start = input$start,
                end = input$end,
                summarize_transcripts = input$summarizeTranscripts
            )
        })

        output$coveragePlot <- renderPlot({
            w$show()

            coverage_return()$plot
        })

        output$coverageTable <- renderDT({
            datatable(coverage_return()$table,
                extensions = "Buttons",
                options = list(
                    dom = "Bft", buttons = c("copy", "csv"),
                    scrollY = "400px"
                )
            )
        })

        output$downloadPlot <- downloadHandler(
            filename = function() {
                paste("coverage", ".pdf", sep = "")
            },
            content = function(file) {
                ggsave(file, coverage_return()$plot, width = 16, height = 12)
            }
        )
    })
}

#' Reformat SingleCellExperiment Object Metadata UI
#'
#' @noRd
reformatMetadataDRui <- function(id) {
    ns <- NS(id)
    tagList(
        chevreulBox(
            title = "Reformat Metadata",
            checkboxInput(ns("header"), "Header", TRUE),
            fileInput(ns("metaFile"),
                "Choose CSV File of metadata with cell names in first column",
                accept = c(
                    "text/csv",
                    "text/comma-separated-values,text/plain",
                    ".csv"
                )
            ),
            actionButton(ns("updateMetadata"), "Update Metadata"),
            radioButtons(ns("updateMethod"), "Update By:",
                choices = c("table (below)" = "spreadsheet", "uploaded file" = "file"), inline = TRUE
            ),
            width = 12,
            dataSelectUI(ns("select1")),
            dataFilterUI(ns("filter1")),
            hidden(actionButton(ns("sync"), label = NULL, icon = icon("sync"))),
            dataOutputUI(ns("output-active")),
            dataOutputUI(ns("output-update"), icon = "file-download"),
            hidden(actionButton(ns("cut"), label = NULL, icon = icon("cut"))),
            dataEditUI(ns("edit1"))
        ) |>
            default_helper(type = "markdown", content = "reformatMetadata")
    )
}

#' Reformat SingleCellExperiment Object Metadata Server
#'
#' @param object a SingleCellExperiment object
#'
#' @noRd
reformatMetadataDR <- function(id, object,
                               featureType = "gene",
                               col_bind = NULL,
                               col_edit = TRUE,
                               col_options = NULL,
                               col_stretch = FALSE,
                               col_names = TRUE,
                               col_readonly = NULL,
                               col_factor = FALSE,
                               row_bind = NULL,
                               row_edit = TRUE,
                               save_as = NULL,
                               title = NULL,
                               logo = NULL,
                               logo_size = 30,
                               logo_side = "left",
                               viewer = "dialog",
                               viewer_height = 800,
                               viewer_width = 2000,
                               theme = "yeti",
                               read_fun = "read_csv",
                               read_args = NULL,
                               write_fun = "write.csv",
                               write_args = NULL,
                               quiet = FALSE,
                               code = FALSE,
                               hide = FALSE) {
    moduleServer(id, function(input, output, session) {
        table_out <- reactive({
            req(object())
            get_colData(object())
        })

        values <- reactiveValues(
            data = NULL, data_active = NULL,
            rows = NULL, columns = NULL, cut = FALSE
        )

        observeEvent(table_out(), {
            values$rows <- NULL
            values$columns <- NULL

            values$data <- table_out() |>
                data_bind_rows(row_bind = row_bind) |>
                data_bind_cols(col_bind = col_bind) |>
                identity()
        })

        data_select <- dataSelectServer("select1",
            data = reactive(values$data),
            hide = hide
        )
        data_filter <- dataFilterServer("filter1",
            data = reactive(values$data),
            hide = hide
        )
        observe({
            values$rows <- data_filter$rows()
            values$columns <- data_select$columns()
        })

        observe({
            if (length(values$rows) == 0 & length(values$columns) == 0) {
                values$data_active <- values$data
            } else {
                if (length(values$rows) != 0 & length(values$columns) == 0) {
                    values$data_active <-
                        values$data[values$rows, , drop = FALSE]
                } else if (length(values$rows) == 0 & length(values$columns) != 0) {
                    values$data_active <-
                        values$data[, values$columns, drop = FALSE]
                } else if (length(values$rows) != 0 & length(values$columns) != 0) {
                    values$data_active <-
                        values$data[values$rows, values$columns, drop = FALSE]
                }
            }
        })

        data_update <- dataEditServer("edit1",
            data = reactive({
                values$data_active
            }),
            col_bind = NULL, col_edit = col_edit, col_options = col_options,
            col_stretch = col_stretch, col_names = col_names,
            col_readonly = col_readonly, col_factor = col_factor,
            row_bind = NULL, row_edit = row_edit, quiet = quiet,
            height = viewer_height, width = viewer_width
        )
        observe({
            values$data_active <- data_update()
        })

        observeEvent(input$updateMetadata, {
            if (input$updateMethod == "file") {
                inFile <- input$metaFile

                if (is.null(inFile)) {
                    return(NULL)
                }

                object(set_colData(object(), read_csv(inFile$datapath)))
            } else if (input$updateMethod == "spreadsheet") {
                reformatted_sce <-
                    propagate_spreadsheet_changes(values$data_active, object())
                object(reformatted_sce)
            }
        })


        observeEvent(input$sync, {
            if (length(values$rows) == 0 & length(values$columns) == 0) {
                values$data <- values$data_active
            } else {
                if (length(values$rows) != 0 & length(values$columns) == 0) {
                    values$data[values$rows, ] <-
                        values$data_active
                } else if (length(values$rows) == 0 & length(values$columns) != 0) {
                    values$data[, values$columns] <-
                        values$data_active
                } else if (length(values$rows) != 0 & length(values$columns) != 0) {
                    values$data[values$rows, values$columns] <-
                        values$data_active
                }
                if (!is.null(values$data_active)) {
                    if (!all(rownames(values$data_active) == rownames(values$data)[values$rows])) {
                        rownames(values$data)[values$rows] <-
                            rownames(values$data_active)
                    }
                    if (!all(colnames(values$data_active) == colnames(values$data)[values$columns])) {
                        colnames(values$data)[values$columns] <-
                            colnames(values$data_active)
                    }
                }
            }
        })

        dataOutputServer("output-active",
            data = reactive({
                values$data_active
            }), save_as = "metadata.csv", write_fun = write_fun,
            write_args = write_args,
            hide = hide
        )
        dataOutputServer("output-update",
            data = reactive({
                values$data
            }), save_as = "metadata.csv", write_fun = write_fun,
            write_args = write_args,
            hide = hide
        )

        # SAVE AS
        if (!hide & !is.null(save_as)) {
            do.call(
                write_fun,
                c(list(x_edit, save_as), write_args)
            )
        }

        observeEvent(input$cut, {
            if (values$cut) {
                values$cut <- FALSE
                updateButton(session, "cut", NULL,
                    block = FALSE,
                    style = "danger"
                )
            } else {
                values$cut <- TRUE
                updateButton(session, "cut", NULL,
                    block = FALSE,
                    style = "success"
                )
            }
        })

        return(object)
    })
}

#' Create a Table of single Cell Projects
#'
#' Uses a list of projects to create a matrix of single cell projects
#'
#' @param proj_list List of projects
#'
#' @return a tibble of single cell projects
create_proj_matrix <- function(proj_list) {
    proj_list <- unlist(proj_list)

    sp_cols <- c("date", "user", "note", "species")
    
    proj_matrix <-
        proj_list |> 
        tibble::enframe("project_name", "project_path") |> 
        tidyr::separate(project_name, sp_cols, sep = "-", remove = FALSE) |> 
        mutate(date = path_file(date)) |>
        dplyr::select(everything(), project_name) |> 
        identity()

    primary_projects <-
        proj_matrix |>
        filter(!grepl("integrated_projects", project_path)) |>
        filter(str_count(project_name, "_") == 1) |>
        identity()

    integrated_projects <-
        proj_matrix |>
        anti_join(primary_projects) |>
        identity()

    proj_matrices <- list(primary_projects = primary_projects, integrated_projects = integrated_projects)

    return(proj_matrices)
}

#' Subset by new colData
#'
#' Subset the object using new colData
#'
#' @param colData_path Path to new colData
#' @param object A object
#'
#' @return a SingleCellExperiment object
#'
subset_by_colData <- function(colData_path, object) {
    upload_colData <- read_csv(colData_path, col_names = "sample_id") |>
        filter(!is.na(sample_id) & !sample_id == "sample_id") |>
        mutate(name = sample_id) |>
        column_to_rownames("sample_id") |>
        identity()

    upload_cells <- rownames(upload_colData)

    object <- object[, colnames(object) %in% upload_cells]

    colData(object) <- merge(colData(object), upload_colData, by = "row.names")

    return(object)
}


#' Read in Gene and Transcript SingleCellExperiment Objects
#'
#' @param proj_dir path to project directory
#' @param prefix default "unfiltered"
#'
#' @return a SingleCellExperiment object
load_alabaster_path <- function(proj_dir = getwd(), prefix = "unfiltered") {
    alabaster_regex <- paste0(paste0(".*/", prefix, "_alabaster"))

    alabaster_path <- path(proj_dir, "output", "singlecellexperiment") |>
        dir_ls(regexp = alabaster_regex)

    if (!length(alabaster_path) == 0) {
        return(alabaster_path)
    }

    stop(alabaster_path, " does not exist in current working directory ", getwd(), ".",
        call. = FALSE
    )
}

#' Load SingleCellExperiment Files from a single project path
#'
#' @param proj_dir project directory
#' @param ... extra args passed to load_alabaster_path
#'
#' @return a SingleCellExperiment object
load_alabaster_from_proj <- function(proj_dir, ...) {
    alabaster_file <- load_alabaster_path(proj_dir, ...)
    alabaster_file <- readObject(alabaster_file)
}

#' Create a minimal chevreulShiny app using SingleCellExperiment input
#'
#' @param single_cell_sce a singlecell object
#' @param appTitle a title for the app
#' @param organism_type human or mouse
#' @param futureMb the megabytes available for the future package
#' @param db_name a database of bigwig files
#'
#' @return a minimal chevreulShiny app
#' @export
#'
#' @examples
#' 
#' if (interactive() ) {
#'   data("tiny_sce")
#'   minimalChevreulApp(tiny_sce)
#' }
#' 
minimalChevreulApp <- function(single_cell_sce = NULL,
                          appTitle = NULL,
                          organism_type = "human",
                          futureMb = 13000,
                          db_name = "single-cell-projects.db") {

  db_path <- file.path(user_cache_dir(appname="chevreul"), db_name)

    plan(strategy = "multicore", workers = 6)
    future_size <- futureMb * 1024^2
    options(future.globals.maxSize = future_size)
    options(shiny.maxRequestSize = 40 * 1024^2)
    options(DT.options = list(
        pageLength = 2000, paging = FALSE,
        info = TRUE, searching = TRUE, autoWidth = FALSE, ordering = TRUE,
        scrollX = TRUE, language = list(search = "Filter:")
    ))
    header <- dashboardHeader(title = appTitle)
    sidebar <- dashboardSidebar(
        textOutput("appTitle"),
        uiOutput("featureType"),
        sidebarMenu(
            menuItem("Overview Plots",
                tabName = "comparePlots", icon = icon("chart-bar"),
                selected = TRUE
            ), menuItem("Violin",
                tabName = "violinPlots", icon = icon("sort")
                ), menuItem("Differential Expression",
                tabName = "diffex", icon = icon("magnet")
            ), menuItem("Find Markers",
                tabName = "findMarkers", icon = icon("bullhorn")
            ), menuItem("Heatmap",
                       tabName = "heatmapPlots", icon = icon("square")
            ), menuItem("Transcripts",
                        tabName = "allTranscripts", icon = icon("sliders-h")
            ), menuItem("Subset",
                tabName = "subsetObject", icon = icon("filter")
            ),  menuItem("Regression",
                tabName = "regressFeatures", icon = icon("eraser")
            ),menuItem("Formatting",
                       tabName = "reformatMetadata", icon = icon("columns")
            ), menuItem("Coverage Plots",
                        tabName = "coveragePlots", icon = icon("mountain")
            ), menuItem("Technical Information",
                tabName = "techInfo", icon = icon("cogs")
            )
        ),
        actionButton("changeEmbedAction",
            label = "Change Embedding Parameters"
        ),
        changeEmbedParamsui("changeembed"),
        width = 250
    )
    body <- dashboardBody(
        use_waiter(),
        tabItems(
            tabItem(
                tabName = "comparePlots",
                h2("Overview Plots") |>
                    default_helper(type = "markdown", 
                                   content = "comparePlots"),
                plotDimRedui("plotdimred1"),
                plotDimRedui("plotdimred2"),
                plotReadCountui("plotreadcount1"),
                plotReadCountui("plotreadcount2"),
                chevreulBox(
                    title = "Selected Cells",
                    tableSelectedui("tableselected"),
                    width = 6
                ),
                plotClustree_UI("clustreePlot")
            ),
            tabItem(
                tabName = "violinPlots",
                fluidRow(
                    plotViolinui("violinPlot")
                )
            ),
            tabItem(
                tabName = "heatmapPlots",
                fluidRow(
                    plotHeatmapui("heatMap")
                )
            ),
            tabItem(
                tabName = "coveragePlots",
                fluidRow(
                    plotCoverage_UI("coverageplots")
                )
            ),
            tabItem(
                tabName = "reformatMetadata",
                fluidRow(
                    reformatMetadataDRui("reformatMetadataDR")
                )
            ),
            tabItem(
                tabName = "subsetObject",
                h2("Subset Object Input") |>
                    default_helper(type = "markdown", 
                                   content = "subsetObject"),
                plotDimRedui("subset"),
                chevreulBox(
                    title = "Subset Settings",
                    checkboxInput("legacySettingsSubset",
                                  "Use Legacy Settings", value = FALSE),
                    actionButton("subsetAction",
                                 "subset object by selected cells"),
                    actionButton("subsetCsv", "subset object by uploaded csv"),
                    fileInput("uploadCsv",
                        "Upload .csv file with cells to include",
                        accept = c(".csv")
                    ),
                    useShinyjs(),
                    # runcodeUI(code = "alert('Hello!')"),
                    textOutput("subsetMessages"),
                    width = 6
                ),
                chevreulBox(
                    title = "Selected Cells", tableSelectedui("subset"),
                    width = 6
                )
            ), tabItem(
                tabName = "findMarkers",
                h2("Find Markers"),
                chevreulMarkersui("findmarkers"),
                plotDimRedui("markerScatter")
            ), tabItem(
                tabName = "allTranscripts",
                h2("All Transcripts"),
                plotDimRedui("alltranscripts2"),
                allTranscriptsui("alltranscripts1")
            ),
            tabItem(
                tabName = "diffex",
                h2("Differential Expression") |>
                    default_helper(type = "markdown", content = "diffex"),
                plotDimRedui("diffex"),
                diffexui("diffex")
            ),
            tabItem(
                tabName = "regressFeatures",
                fluidRow(
                    chevreulBox(
                        title = "Regress Features",
                        actionButton(
                            "regressAction",
                            "Regress Objects By Genes"
                        ),
                        width = 12
                    ) |>
                        default_helper(type = "markdown",
                                       content = "regressFeatures")
                )),
            tabItem(
                tabName = "techInfo",
                h2("Technical Information"),
                techInfoui("techInfo")
            )
        )
    )

    ui <- function(request) {
        ui <- dashboardPage(
            header = header, sidebar = sidebar,
            body = body
        )
    }
    server <- function(input, output, session) {
        w <- Waiter$new()

        observe_helpers(
            help_dir = system.file("helpers", package = "chevreulShiny"))
        options(warn = -1)

        object <- reactiveVal(NULL)
        observe({
            req(!is.null(single_cell_sce))
            object(single_cell_sce)
        })

        organism_type <- reactive({
            "human"
        })

        plot_types <- reactive({
            req(!is.null(object()))
            list_plot_types(object())
        })

        featureType <- reactive({
            "gene"
            # input$feature_type
        })


        observe({
            reformatted_sce <- reformatMetadataDR( 
                                             "reformatMetadataDR", 
                                             object, featureType)
            object(reformatted_sce())
        })

        reductions <- reactive({
            req(!is.null(object()))
            reducedDimNames(object())
        })

        observe({
            req(!is.null(object()))

            plotDimRed("plotdimred1", object, plot_types, featureType,
                organism_type = organism_type, reductions
            )
            plotDimRed("plotdimred2", object, plot_types, 
                       featureType,
                organism_type = organism_type, reductions
            )
            plotDimRed("diffex", object, plot_types, featureType,
                organism_type = organism_type, reductions
            )
            plotDimRed("subset", object, plot_types, featureType,
                organism_type = organism_type, reductions
            )
            plotDimRed("markerScatter", object, plot_types, 
                       featureType,
                organism_type = organism_type, reductions
            )
            plotReadCount("plotreadcount1", object, plot_types)
            plotReadCount("plotreadcount2", object, plot_types)
        })
        plotViolin("violinPlot", object, featureType,
            organism_type
        )
        plotHeatmap("heatMap", object, featureType,
            organism_type
        )
        plotCoverage("coverageplots", object, plot_types, proj_dir, 
            organism_type
        )
        plotClustree("clustreePlot", object)
        tableSelected("tableselected", object)
        diffex_selected_cells <- tableSelected("diffex",
            object
        )
        subset_selected_cells <- tableSelected("subset",
            object
        )

        observeEvent(input$changeEmbedAction, {
            showModal(modalDialog(
                title = "Recalculating Embedding",
                "This process may take a minute or two!"
            ))
            object <- changeEmbedParams( "changeembed",
                object
            )
            removeModal()
        })

        chevreulMarkers("findmarkers", object, plot_types, 
                   featureType)

        # plot all transcripts
        observe({
            req(featureType())
            req(!is.null(object()))
            allTranscripts( "alltranscripts1", object, featureType,
                organism_type
            )

            plotDimRed( "alltranscripts2", object, plot_types, 
                       featureType,
                organism_type = organism_type, reductions
            )
        })

        diffex("diffex", object, featureType,
            diffex_selected_cells
        )
        
        observeEvent(input$subsetAction, {
            req(input$subsetAction)
            req(subset_selected_cells())
            withCallingHandlers(
                {
                    html("subsetMessages", "")
                    message("Beginning")
                    subset_sce <- 
                        object()[, colnames(object()) %in% 
                                     subset_selected_cells()]
                    object(subset_sce)
                    if (length(unique(object()$batch)) > 1) {
                        message("reintegrating gene expression")
                        reintegrated_sce <- reintegrate_sce(object(),
                            resolution = seq(0.2, 1, by = 0.2),
                            organism = metadata(object())$experiment$organism
                        )
                        object(reintegrated_sce)
                    } else {
                        processed_sce <- 
                            sce_process(object(), 
                                            resolution = seq(0.2, 1, by = 0.2))
                        object(processed_sce)
                    }
                    message("Complete!")
                },
                message = function(m) {
                    html(id = "subsetMessages", html = paste0(
                        "Subsetting Object: ",
                        m$message
                    ), add = FALSE)
                }
            )
        })
        observeEvent(input$subsetCsv, {
            req(input$subsetCsv)
            req(input$uploadCsv)
            withCallingHandlers(
                {
                    html("subsetMessages", "")
                    message("Beginning")
                    subset_sce <- subset_by_colData(
                        input$uploadCsv$datapath,
                        object()
                    )
                    object(subset_sce)

                    if (length(unique(object()[["batch"]])) > 1) {
                        message("reintegrating gene expression")
                        reintegrated_sce <- reintegrate_sce(object(),
                            resolution = seq(0.2, 1, by = 0.2),
                            organism = metadata(object())$experiment$organism
                        )
                        object(reintegrated_sce)
                    } else {
                        processed_sce <- 
                            sce_process(object(), 
                                           resolution = seq(0.2, 1, by = 0.2))
                        object(processed_sce)
                    }
                    message("Complete!")
                },
                message = function(m) {
                    html(id = "subsetMessages", html = paste0(
                        "Subsetting Object: ",
                        m$message
                    ), add = FALSE)
                }
            )
        })

        observeEvent(input$regressAction, {
            req(!is.null(object()))
            showModal(modalDialog(
                title = "Regressing out cell cycle effects",
                "This process may take a minute or two!"
            ))
            regressed_sce <- regress_cell_cycle(object())
            object(regressed_sce)
            removeModal()
        })

        techInfo("techInfo", object)
    }
    shinyApp(ui, server, enableBookmarking = "server")
}

#' Unite metadata
#'
#' @param object A SingleCellExperiment object
#' @param group_bys A feature or variable to combine
#'
#' @return a SingleCellExperiment object with Idents formed from concatenation of group_bys
#' @export
#'
#' @examples
#' 
#' 
#' data(small_example_dataset)
#' unite_metadata(small_example_dataset, "Mutation_Status")
#'
unite_metadata <-
    function(object, group_bys) {
        newcolname <- paste(group_bys, collapse = "_by_")
        newdata <- colData(object)[group_bys] |>
            as.data.frame() |>
            unite(!!newcolname, any_of(group_bys)) |>
            deframe()
        
        return(object)
    }

#' Get Transcripts in object
#'
#' Get transcript ids in objects for one or more gene of interest
#'
#' @param object A SingleCellExperiment object
#' @param gene Gene of interest
#'
#' @return transcripts constituting a 
#' gene of interest in a SingleCellExperiment object
get_transcripts_from_sce <- function(object, gene) {
    transcripts <- genes_to_transcripts(gene)

    transcripts <- transcripts[transcripts %in% get_features(object, 
                                                             "transcript")]
}


#' Create a database of chevreulShiny projects
#'
#' Create a database containing chevreulShiny projects
#'
#' @param cache_location Path to cache "~/.cache/chevreul"
#' @param sqlite_db Database to be created
#' @param verbose print messages
#'
#' @return a sqlite database with SingleCellExperiment objects
create_project_db <- function(cache_location = "~/.cache/chevreul", 
                              sqlite_db = "single-cell-projects.db", 
                              verbose = TRUE) {
    if (!dir.exists(cache_location)) {
        dir.create(cache_location)
    }
    con <- dbConnect(SQLite(), path(cache_location, sqlite_db))
    projects_tbl <- tibble(project_name = character(), 
                           project_path = character(), 
                           project_slug = character(), 
                           project_type = character(), )
    message(paste0(
        "building table of chevreulShiny projects at ", path(cache_location, 
        sqlite_db)))
    tryCatch({
        dbWriteTable(con, "projects_tbl", projects_tbl)
    }, warning = function(w) {
        message(sprintf("Warning in %s: %s", deparse(w[["call"]]), 
                        w[["message"]]))
    }, error = function(e) {
        message("projects db already exists!")
    }, finally = {
    })
    dbDisconnect(con)
}

#' Update a database of chevreulShiny projects
#'
#' Add new/update existing projects to the database by recursing fully
#'
#' @param projects_dir The project directory to be updated
#' @param cache_location Path to cache "~/.cache/chevreul"
#' @param sqlite_db sqlite db
#' @param verbose print messages
#'
#' @return a sqlite database with SingleCellExperiment objects
update_project_db <- function(
        projects_dir = NULL,
        cache_location = "~/.cache/chevreul",
        sqlite_db = "single-cell-projects.db",
        verbose = TRUE) {
    if (!dir.exists(cache_location)) {
        dir.create(cache_location)
    }

    con <- dbConnect(SQLite(), path(cache_location, sqlite_db))
    
    projects_tbl <-
        dir_ls(projects_dir, glob = "*.here", recurse = TRUE, 
               fail = FALSE, all = TRUE) |>
               path_dir() 
        
        names(projects_tbl) <- path_file(projects_tbl)
        
        projects_tbl <- 
        projects_tbl |>
        enframe("project_name", "project_path") |>
        mutate(project_slug = str_remove(project_name, "_proj$")) |>
        mutate(project_type = path_file(path_dir(project_path))) |>
        identity()

    current_projects_tbl <-
        dbReadTable(con, "projects_tbl") |>
        filter(file.exists(project_path)) |>
        filter(!project_path %in% projects_tbl$project_path) |>
        bind_rows(projects_tbl) |>
        distinct(project_path, .keep_all = TRUE)

    dbWriteTable(con, "projects_tbl", projects_tbl, overwrite = TRUE)

    dbDisconnect(con)
}

#' Update a database of chevreulShiny projects
#'
#' Append projects to database
#'
#' @param new_project_path new project path
#' @param cache_location Path to cache "~/.cache/chevreul"
#' @param sqlite_db sqlite db
#' @param verbose print messages
#' @return a sqlite database with SingleCellExperiment objects

append_to_project_db <- function(
        new_project_path,
        cache_location = "~/.cache/chevreul",
        sqlite_db = "single-cell-projects.db",
        verbose = TRUE) {
    if (!dir.exists(cache_location)) {
        dir.create(cache_location)
    }

    con <- dbConnect(SQLite(), path(cache_location, sqlite_db))

    names(new_project_path) <- path_file(new_project_path)

    projects_tbl <-
        new_project_path |>
        enframe("project_name", "project_path") |>
        mutate(project_slug = str_remove(project_name, "_proj$")) |>
        mutate(project_type = path_file(path_dir(project_path))) |>
        identity()

    current_projects_tbl <-
        dbReadTable(con, "projects_tbl") |>
        filter(file.exists(project_path)) |>
        filter(!project_path %in% projects_tbl$project_path) |>
        bind_rows(projects_tbl) |>
        distinct(project_path, .keep_all = TRUE)

    dbWriteTable(con, "projects_tbl", current_projects_tbl, overwrite = TRUE)

    dbDisconnect(con)
}

#' Read a database of chevreulShiny projects
#'
#' Reads database of chevreulShiny projects to a data frame
#'
#' @param cache_location Path to cache "~/.cache/chevreul"
#' @param sqlite_db sqlite db
#' @param verbose print messages
#'
#' @return a tibble with SingleCellExperiment objects
#

read_project_db <- function(
        cache_location = "~/.cache/chevreul",
        sqlite_db = "single-cell-projects.db",
        verbose = TRUE) {
    if (!dir.exists(cache_location)) {
        dir.create(cache_location)
    }

    con <- dbConnect(SQLite(), path(cache_location, sqlite_db))

    current_projects_tbl <-
        dbReadTable(con, "projects_tbl")

    dbDisconnect(con)

    return(current_projects_tbl)
}

#' Make Bigwig Database
#'
#'
#' @param new_project Project directory
#' @param cache_location Path to cache "~/.cache/chevreul"
#' @param sqlite_db sqlite db containing bw files
#'
#' @return a sqlite database of bigwig files for cells 
#' in a SingleCellExperiment object
make_bigwig_db <- function(new_project = NULL, 
                           cache_location = "~/.cache/chevreul/", 
                           sqlite_db = "bw-files.db") {
    new_bigwigfiles <- dir_ls(new_project, glob = "*.bw", recurse = TRUE)

    names(new_bigwigfiles) <- path_file(new_bigwigfiles)
        
    new_bigwigfiles <- 
    new_bigwigfiles |>
        enframe("name", "bigWig") |>
        mutate(sample_id = 
                   str_remove(name, "_Aligned.sortedByCoord.out.*bw$")) |>
        filter(!str_detect(name, "integrated")) |>
        distinct(sample_id, .keep_all = TRUE) |>
        identity()

    con <- dbConnect(SQLite(), dbname = path(cache_location, sqlite_db))

    all_bigwigfiles <-
        dbReadTable(con, "bigwigfiles") |>
        bind_rows(new_bigwigfiles)

    dbWriteTable(con, "bigwigfiles", all_bigwigfiles, overwrite = TRUE)

    return(all_bigwigfiles)
}

#' Retrieve Metadata from Batch
#'
#' @param batch batch
#' @param projects_dir path to project dir
#' @param db_path path to .db file
#'
#' @return a tibble with cell level metadata from a SingleCellExperiment object
metadata_from_batch <- function(
        batch, projects_dir = "/dataVolume/storage/single_cell_projects",
        db_path = "single-cell-projects.db") {
    mydb <- dbConnect(SQLite(), path(projects_dir, db_path))

    projects_tbl <- dbReadTable(mydb, "projects_tbl") |>
        filter(!project_type %in% c("integrated_projects", "resources"))

    dbDisconnect(mydb)

    metadata <-
        projects_tbl |>
        filter(project_slug == batch) |>
        pull(project_path) |>
        path("data") |>
        dir_ls(glob = "*.csv") |>
        identity()
}


#' Save object to <project>/output/sce/<feature>_sce.rds
#'
#' @param object a SingleCellExperiment object
#' @param prefix a prefix for saving
#' @param proj_dir path to a project directory
#'
#' @return a path to an rds file containing a SingleCellExperiment object
#'
#'
#'
save_sce <- function(object, prefix = "unfiltered", proj_dir = getwd()) {
    sce_dir <- path(proj_dir, "output", "singlecellexperiment")

    dir.create(sce_dir)

    sce_path <- path(sce_dir, paste0(prefix, "_sce.rds"))

    message(paste0("saving to ", sce_path))
    saveRDS(object, sce_path)

    return(object)
}

#' Collate list of variables to be plotted
#'
#' @param object a SingleCellExperiment object
#'
#' @return plot_types a list of category_vars or continuous_vars
#' @export
#' @examples
#' 
#' data(small_example_dataset)
#' list_plot_types(small_example_dataset)
list_plot_types <- function(object) {
    meta_types <- tibble(
        vars = colnames(colData(object)),
        var_type = map_chr(colData(object), class),
        num_levels = unlist(map(colData(object), ~ length(unique(.x))))
    )
    
    meta_types <- meta_types |>
        # filter(!grepl("_snn_res", vars)) |>
        mutate(meta_type = case_when(
            var_type %in% c("integer", "numeric") ~ "continuous",
            var_type %in% c("character", "factor", "logical") ~ "category"
        )) |>
        mutate(meta_type = ifelse(meta_type == "continuous" & num_levels < 30, "category", meta_type)) |>
        filter(num_levels > 1) |>
        identity()
    
    continuous_vars <- meta_types |>
        filter(meta_type == "continuous") |>
        pull(vars)
    
    continuous_vars <- c("feature", continuous_vars)
    
    names(continuous_vars) <- 
        str_to_title(str_replace_all(continuous_vars, "[[:punct:]]", " "))
    
    category_vars <- meta_types |>
        filter(meta_type == "category") |>
        pull(vars)
    
    names(category_vars) <-  
        str_to_title(str_replace_all(
            category_vars, 
            "[^[:alnum:][:space:]\\.]", " "))
    
    plot_types <- list(category_vars = category_vars, continuous_vars = continuous_vars)
    
    
    
    return(plot_types)
}

#' Clean Vector of chevreulShiny Names
#'
#' Cleans names of objects provided in a vector form
#'
#' @param myvec A vector of object names
#'
#' @return a clean vector of object names
#' @export
#' @examples
#' 
#' data(small_example_dataset)
#' make_chevreul_clean_names(colnames(
#' get_colData(small_example_dataset)))
make_chevreul_clean_names <- function(myvec) {
    names(myvec) <- 
        myvec |> 
        str_replace_all("[^[:alnum:][:space:]\\.]", " ") |> 
        str_to_title()
    
    return(myvec)
}


#' Plotly settings
#'
#' Change settings of a plotly plot
#'
#' @param plotly_plot  A plotly plot
#' @param width Default set to '600'
#' @param height Default set to '700'
#'
#' @return a plotly plot with settings changed
plotly_settings <- function(plotly_plot, width = 600, height = 700) {
    plotly_plot |>
        layout(dragmode = "lasso") |>
        config(toImageButtonOptions = list(format = "svg", 
                                           filename = "myplot", 
                                           width = width, height = height)) |>
        identity()
}

#' Reformat SingleCellExperiment Object Metadata UI
#'
#' @param id id
#'
#' @return UI to reformat the metadata of a SingleCellExperiment object
#' @noRd
reformatMetadataDRui <- function(id) {
    
    tagList(
        chevreulBox(
            title = "Reformat Metadata",
            checkboxInput("header", "Header", TRUE),
            fileInput("metaFile", 
                      "Choose csv of metadata with cell names in first column",
                accept = c(
                    "text/csv",
                    "text/comma-separated-values,text/plain",
                    ".csv"
                )
            ),
            actionButton("updateMetadata", "Update Metadata"),
            radioButtons("updateMethod", 
                         "Update By:", 
                         choices = c("table (below)" = "spreadsheet", 
                                     "uploaded file" = "file"), inline = TRUE),
            width = 12,
            dataSelectUI("select1"),
            dataFilterUI("filter1"),
            hidden(actionButton("sync", label = NULL, 
                                icon = icon("sync"))),
            dataOutputUI("output-active"),
            dataOutputUI("output-update", icon = "file-download"),
            hidden(actionButton("cut", label = NULL, icon = icon("cut"))),
            dataEditUI("edit1")        ) |>
            default_helper(type = "markdown", content = "reformatMetadata")
    )
}

#' Reformat SingleCellExperiment Object Metadata Server
#'
#' @param object SingleCellExperiment object
#'
#' @noRd
reformatMetadataDR <- function(
        input, output, session, object, featureType = "gene",
        col_bind = NULL,
        col_edit = TRUE,
        col_options = NULL,
        col_stretch = FALSE,
        col_names = TRUE,
        col_readonly = NULL,
        col_factor = FALSE,
        row_bind = NULL,
        row_edit = TRUE,
        save_as = NULL,
        title = NULL,
        logo = NULL,
        logo_size = 30,
        logo_side = "left",
        viewer = "dialog",
        viewer_height = 800,
        viewer_width = 2000,
        theme = "yeti",
        read_fun = "read_csv",
        read_args = NULL,
        write_fun = "write.csv",
        write_args = NULL,
        quiet = FALSE,
        code = FALSE,
        hide = FALSE) {
    

    table_out <- reactive({
        req(object())
        get_colData(object())
    })

    values <- reactiveValues(
        data = NULL, data_active = NULL,
        rows = NULL, columns = NULL, cut = FALSE
    )

    observeEvent(table_out(), {
        values$rows <- NULL
        values$columns <- NULL

        values$data <- table_out() |>
            data_bind_rows(row_bind = row_bind) |>
            data_bind_cols(col_bind = col_bind) |>
            identity()
    })

    data_select <- dataSelectServer("select1",
        data = reactive(values$data),
        hide = hide
    )
    data_filter <- dataFilterServer("filter1",
        data = reactive(values$data),
        hide = hide
    )
    observe({
        values$rows <- data_filter$rows()
        values$columns <- data_select$columns()
    })

    observe({
        if (length(values$rows) == 0 & length(values$columns) == 0) {
            values$data_active <- values$data
        } else {
            if (length(values$rows) != 0 & 
                length(values$columns) == 0) {
                values$data_active <- 
                    values$data[values$rows, , drop = FALSE]
            } else if (length(values$rows) == 0 & 
                       length(values$columns) != 0) {
                values$data_active <- 
                    values$data[, values$columns, drop = FALSE]
            } else if (length(values$rows) != 0 & 
                       length(values$columns) != 0) {
                values$data_active <- 
                    values$data[values$rows, values$columns, drop = FALSE]
            }
        }
    })

    data_update <- dataEditServer("edit1",
        data = reactive({
            values$data_active
        }),
        col_bind = NULL, col_edit = col_edit, col_options = col_options,
        col_stretch = col_stretch, col_names = col_names,
        col_readonly = col_readonly, col_factor = col_factor,
        row_bind = NULL, row_edit = row_edit, quiet = quiet, 
        height = viewer_height, width = viewer_width
    )
    observe({
        values$data_active <- data_update()
    })

    observeEvent(input$updateMetadata, {
        if (input$updateMethod == "file") {
            inFile <- input$metaFile

            if (is.null(inFile)) {
                return(NULL)
            }

            object(set_colData(object(), read_csv(inFile$datapath)))
        } else if (input$updateMethod == "spreadsheet") {
            object(propagate_spreadsheet_changes(values$data_active, object()))
        }
    })


    observeEvent(input$sync, {
        if (length(values$rows) == 0 & length(values$columns) == 0) {
            values$data <- values$data_active
        } else {
            if (length(values$rows) != 0 & 
                length(values$columns) == 0) {
                values$data[values$rows, ] <- 
                    values$data_active
            } else if (length(values$rows) == 0 & 
                       length(values$columns) != 0) {
                values$data[, values$columns] <- 
                    values$data_active
            } else if (length(values$rows) != 0 & 
                       length(values$columns) != 0) {
                values$data[values$rows, values$columns] <- 
                    values$data_active
            }
            if (!is.null(values$data_active)) {
                if (!all(rownames(values$data_active) == 
                         rownames(values$data)[values$rows])) {
                    rownames(values$data)[values$rows] <- 
                        rownames(values$data_active)
                }
                if (!all(colnames(values$data_active) == 
                         colnames(values$data)[values$columns])) {
                    colnames(values$data)[values$columns] <- 
                        colnames(values$data_active)
                }
            }
        }
    })

    dataOutputServer("output-active",
        data = reactive({
            values$data_active
        }), save_as = "metadata.csv", write_fun = write_fun, 
        write_args = write_args,
        hide = hide
    )
    dataOutputServer("output-update",
        data = reactive({
            values$data
        }), save_as = "metadata.csv", write_fun = write_fun, 
        write_args = write_args,
        hide = hide
    )

    # SAVE AS
    if (!hide && !is.null(save_as)) {
        do.call(
            write_fun,
            c(list(x_edit, save_as), write_args)
        )
    }

    observeEvent(input$cut, {
        if (values$cut) {
            values$cut <- FALSE
            updateButton(session, "cut", NULL,
                block = FALSE,
                style = "danger"
            )
        } else {
            values$cut <- TRUE
            updateButton(session, "cut", NULL,
                block = FALSE,
                style = "success"
            )
        }
    })

    return(object)
}

#' Add new columns to data
#'
#' @noRd
data_bind_cols <- function(data = NULL,
    col_bind = NULL) {
    # BIND COLUMNS
    if (!is.null(data)) {
        # COLUMNS
        if (!is.null(col_bind)) {
            # NEW COLUMNS
            if (is.null(dim(col_bind))) {
                # COLUMNS AS LIST
                if (inherits(col_bind, "list")) {
                    # NAMES
                    if (is.null(names(col_bind))) {
                        names(col_bind) <- paste0("V", length(col_bind))
                    }
                    # LENGTHS
                    ind <- which(!unlist(lapply(col_bind, length)) == 
                                     nrow(data))
                    if (length(ind) > 0) {
                        for (z in ind) {
                            col_bind[[z]] <- rep(col_bind[[z]], nrow(data))
                        }
                    }
                    # MATRIX
                    col_bind <- do.call("cbind", col_bind)
                    # COLUMN NAMES
                } else {
                    col_bind <- matrix(
                        rep("", nrow(data) * length(col_bind)),
                        ncol = length(col_bind),
                        dimnames = list(
                            rownames(data),
                            col_bind
                        )
                    )
                }
            }
            # BIND NEW COLUMNS
            data <- cbind(
                data,
                col_bind[seq_len(nrow(data)), , drop = FALSE]
            )
        }
    }

    return(data)
}

#' Add new rows to data
#'
#' @noRd
data_bind_rows <- function(data = NULL,
    row_bind = NULL) {
    # BIND ROWS
    if (!is.null(data)) {
        # ROWS
        if (!is.null(row_bind)) {
            # NEW ROWS
            if (is.null(dim(row_bind))) {
                # ROWS AS LIST
                if (inherits(row_bind, "list")) {
                    # NAMES NOT NECESSARY
                    # LENGTHS
                    ind <- which(!unlist(lapply(row_bind, length)) == 
                                     ncol(data))
                    if (length(ind) > 0) {
                        for (z in ind) {
                            row_bind[[z]] <- rep(row_bind[[z]], ncol(data))
                        }
                    }
                    # MATRIX
                    row_bind <- do.call("rbind", row_bind)
                    # ROW NAMES
                } else {
                    row_bind <- matrix(
                        rep("", ncol(data) * length(row_bind)),
                        nrow = length(row_bind),
                        dimnames = list(
                            row_bind,
                            colnames(data)
                        )
                    )
                }
            }
            # BIND NEW ROWS
            data <- rbind(data, row_bind[, seq_len(ncol(data))])
        }
    }

    return(data)
}

#' Default Shiny Helper
#'
#' @param ui_element a shiny ui element
#' @param title a title for the helper
#' @param content content from an rmarkdown file
#' @param type style of the helper
#' @param ... extra params to helper
#'
#' @return a shinyhelper object
#'
#' @noRd
default_helper <- function(ui_element, title = "", 
                           content = "test", type = "inline", ...) {
    helper(ui_element,
        type = type,
        title = title,
        content = content,
        ...
    )
}

#' custom collapsible box
#'
#' @param content interactive content
#' @param title a title for the UI box
#'
#' @return a ui box with standard parameters
#'
#' @noRd
chevreulBox <- function(title = "", ...) {
    box(
        title = title,
        status = "primary",
        solidHeader = TRUE,
        collapsible = TRUE,
        ...
    )
}

#' Custom Dropdown Button
#'
#' @param content interactive content
#' @param title a title
#' @param ...
#'
#' @return a dropdown button
#'
#' @importFrom shinyWidgets dropdownButton
#'
#' @noRd
chevreulDropDownButton <- function(inputId, ...) {
    tagList(
        dropdownButton(
            inputId = inputId,
            ...,
            circle = TRUE, status = "info",
            icon = icon("bars"), width = "300px"
        ),
        br()
    )
}

`%||%` <- function(x, y) {
    if (length(x) <= 0L) {
        y
    } else {
        x
    }
}

`%|||%` <- function(x, y) {
    if (is.null(x)) {
        y
    } else {
        x
    }
}

`%||NA%` <- function(x, y) {
    if (anyNA(x)) {
        y
    } else {
        x
    }
}

`%||nf%` <- function(x, y) {
    if (length(x) <= 0L || anyNA(x)) {
        y
    } else {
        x
    }
}

if_any <- function(condition, x, y) {
    if (any(condition)) {
        x
    } else {
        y
    }
}

#' Human transcripts to genes
#'
#' Lookup table for converting Human (*Homo sapiens*)
#' Ensembl transcript IDs to gene IDs based on genome assembly
#' GRCH38 from Ensembl.
#'
#' @docType data
#' @keywords datasets
#'
#' @details
#' Variables:
#'
#' - enstxp
#' - ensgene
#'
#' @source \url{http://ensembl.org/homo_sapiens}
#'
#' @examples
#' data(grch38_tx2gene)
#' head(grch38_tx2gene)
"grch38_tx2gene"

#' Small example SingleCellExperiment
#'
#' created with scuttle::mockSCE
#'
#' @format An SCE with 200 cells and 1000 genes
#' @source scuttle::mockSCE
"small_example_dataset"

#' Tiny example SingleCellExperiment
#'
#' subset to only NRL from chevreuldata::human_gene_transcript_sce()
#'
#' @format An SCE with only expression of NRL gene and NRL transripts
#' @source chevreuldata::human_gene_transcript_sce()
"tiny_sce"

#' Gene Homologs Between Human and Mouse
#'
#' Homologs drawn from Biomart
#'
#' @format A data frame with 23188 rows and 2 columns
#' \describe{
#'   \item{HGNC.symbol}{human gene symbols}
#'   \item{MGI.symbol}{mouse gene symbols}
#'   ...
#' }
#' @source bioMart
"human_to_mouse_homologs"

#' Cyclone cell cycle pairs by symbol
#'
#' cell cycle genes with paired expression represented by HGNC symbol
#'
#' @format a list of dataframes with G1, G2, and S gene expression
#' \describe{
#'   \item{G1}{G1 gene symbols}
#'   \item{G2}{G2 gene symbols}
#'   \item{S}{S gene symbols}
#'   ...
#' }
#' @source cyclone
"cc.genes.cyclone"

utils::globalVariables(c(".", "Cluster", "DefaultAssay", "FDR", "Gene.Name", "assay", "coverage", "group", "HGNC.symbol", "Idents", "MGI.symbol", "RunUMAP", "SID", "addPerCellQC", "algorithm", "bigWig", "cc.genes", "cc.genes.cyclone", "cluster", "colourInput", "colour_group", "condition", "dashboardPage", "db", "end.umap_1", "end.umap_2", "enriched_pathways_by_cluster_info", "enstxp", "feature", "format_pathway_table", "gene_index", "getEnrichedPathways", "gg_fill_hue group", "human_to_mouse_homologs", "meta_type", "name", "num_levels", "obect_pipeline", "object", "plotUMAP", "plot_ridge", "precedence", "proj_dir", "project_name", "project_path", "project_slug", "project_type", "rn", "sample_id", "start.umap_1", "start.umap_2", "summary.logFC", "symbol", "transcript", "updateButton", "write_csv", "x", "x_edit", "xend", "y", "yend", "DataFrame", "gg_fill_hue", "grch38", "grch38_tx2gene", "grcm38", "grcm38_tx2gene", "bigwig_db", "colData<-"))

#' Human annotation data
#'
#' Human (*Homo sapiens*) annotations based on
#' genome assembly GRCH38 from Ensembl.
#'
#' @docType data
#' @keywords datasets
#'
#' @details
#' Variables:
#'
#' - ensgene
#' - entrez
#' - symbol
#' - chr
#' - start
#' - end
#' - strand
#' - biotype
#' - description
#'
#' @source \url{http://ensembl.org/homo_sapiens}
#'
#' @examples
#' data("grch38")
#' head(grch38)
"grch38"

#' Prep Slider Values
#'
#' @param default_val Provide a default value
#'
#' @noRd
prep_slider_values <- function(default_val) {
    min <- round(default_val * 0.25, digits = 1)
    max <- round(default_val * 2.0, digits = 1)
    step <- 10^((ceiling(log10(default_val))) - 1)
    value <- default_val
    return(list(min = min, max = max, value = value, step = step))
}


#' Create a shiny app for a project on disk
#'
#' @param preset_project A preloaded project to start the app with
#' @param appTitle A title of the App
#' @param organism_type human or mouse
#' @param futureMb amount of Mb allocated to future package
#' @param db_name sqlite database with list of saved
#' SingleCellExperiment objects
#' @return a shiny app
chevreulApp <-
  function(preset_project,
           appTitle = "chevreul",
           organism_type = "human",
           futureMb = 13000,
           db_name = "single-cell-projects.db") {

    db_path <- file.path(user_cache_dir(appname="chevreulShiny"), db_name)

    message(packageVersion("chevreulShiny"))
    plan(strategy = "multicore", workers = 6)
    future_size <- futureMb * 1024^2
    options(future.globals.maxSize = future_size)
    options(shiny.maxRequestSize = 40 * 1024^2)
    options(DT.options = list(
        pageLength = 2000, paging = FALSE,
        info = TRUE, searching = TRUE, autoWidth = FALSE, ordering = TRUE,
        scrollX = TRUE, language = list(search = "Filter:")
    ))
    header <- dashboardHeader(title = appTitle)
    sidebar <- dashboardSidebar(
        uiOutput("projInput"),
        actionButton("loadProject", "Load Selected Project") |>
            default_helper(type = "markdown", content = "overview"),
        textOutput("appTitle"),
        bookmarkButton(),
        prettyRadioButtons("organism_type",
            inline = TRUE,
            "Organism", choices = c("human", "mouse"), selected = organism_type
        ),
        shinyFilesButton("objectUpload", "Load a SingleCellExperiment Dataset",
            "Please select a .rds file",
            multiple = FALSE
        ),
        shinySaveButton("saveSCE", "Save Current Dataset",
            "Save file as...",
            filetype = list(rds = "rds")
        ),
        verbatimTextOutput("savefile"),
        sidebarMenu(
            menuItem("Integrate Projects",
                tabName = "integrateProjects", icon = icon("object-group")
            ), menuItem("Reformat Metadata",
                tabName = "reformatMetadataDR", icon = icon("columns")
            ), menuItem("Plot Data",
                tabName = "comparePlots", icon = icon("chart-bar"),
                selected = TRUE
            ), menuItem("Heatmap/Violin Plots",
                tabName = "violinPlots", icon = icon("sort")
            ), menuItem("Coverage Plots",
                tabName = "coveragePlots", icon = icon("mountain")
            ), menuItem("Differential Expression",
                tabName = "diffex", icon = icon("magnet")
            ), menuItem("Find Markers",
                tabName = "findMarkers", icon = icon("bullhorn")
            ), menuItem("Subset SingleCellExperiment Input",
                tabName = "subsetSingleCellExperiment", icon = icon("filter")
            ), menuItem("All Transcripts",
                tabName = "allTranscripts", icon = icon("sliders-h")
            ), menuItem("RNA Velocity",
                tabName = "plotVelocity", icon = icon("tachometer-alt")
            ), menuItem("Regress Features",
                tabName = "regressFeatures", icon = icon("eraser")
            ), menuItem("Technical Information",
                tabName = "techInfo", icon = icon("cogs")
            )
        ),
        actionButton("changeEmbedAction",
            label = "Change Embedding Parameters"
        ),
        changeEmbedParamsui("changeembed"),
        width = 250
    )
    body <- dashboardBody(
        use_waiter(),
        tabItems(
            tabItem(
                tabName = "comparePlots",
                h2("Compare Plots") |>
                    default_helper(type = "markdown", 
                                   content = "comparePlots"),
                plotDimRedui("plotdimred1"),
                plotDimRedui("plotdimred2"),
                plotReadCountui("plotreadcount1"),
                plotReadCountui("plotreadcount2"),
                chevreulBox(
                    title = "Selected Cells",
                    tableSelectedui("tableselected"),
                    width = 6
                ),
                plotClustree_UI("clustreePlot")
            ),
            tabItem(
                tabName = "violinPlots",
                fluidRow(
                    plotHeatmapui("heatMap")
                ),
                fluidRow(
                    plotViolinui("violinPlot")
                )
            ),
            tabItem(
                tabName = "coveragePlots",
                fluidRow(
                    plotCoverage_UI("coverageplots")
                )
            ),
            tabItem(
                tabName = "integrateProjects",
                fluidRow(
                    integrateProjui("integrateproj"),
                    width = 12
                )
            ),
            tabItem(
                tabName = "reformatMetadataDR",
                fluidRow(
                    reformatMetadataDRui("reformatMetadataDR")
                )
            ),
            tabItem(
                tabName = "subsetSingleCellExperiment",
                h2("Subset SingleCellExperiment Input") |>
                    default_helper(type = "markdown",
                                   content = "subsetSingleCellExperiment"),
                plotDimRedui("subset"),
                chevreulBox(
                    title = "Subset Settings",
                    checkboxInput("legacySettingsSubset", 
                                  "Use Legacy Settings",
                                  value = FALSE),
                    actionButton("subsetAction",
                                 "subset object by selected cells"),
                    actionButton("subsetCsv", "subset object by uploaded csv"),
                    fileInput("uploadCsv",
                        "Upload .csv file with cells to include",
                        accept = c(".csv")
                    ),
                    useShinyjs(),
                    # runcodeUI(code = "alert('Hello!')"),
                    textOutput("subsetMessages"),
                    width = 6
                ),
                chevreulBox(
                    title = "Selected Cells", tableSelectedui("subset"),
                    width = 6
                )
            ), tabItem(
                tabName = "findMarkers",
                h2("Find Markers"),
                chevreulMarkersui("findmarkers"),
                plotDimRedui("markerScatter")
            ), tabItem(
                tabName = "allTranscripts",
                h2("All Transcripts"),
                plotDimRedui("alltranscripts2"),
                allTranscriptsui("alltranscripts1")
            ),
            tabItem(
                tabName = "diffex",
                h2("Differential Expression") |>
                    default_helper(type = "markdown", content = "diffex"),
                plotDimRedui("diffex"),
                diffexui("diffex")
            ),
            tabItem(
                tabName = "regressFeatures",
                fluidRow(
                    chevreulBox(
                        title = "Regress Features",
                        actionButton(
                            "regressAction",
                            "Regress SingleCellExperiment Objects By Genes"
                        ),
                        width = 12
                    ) |>
                        default_helper(type = "markdown",
                                       content = "regressFeatures")
                )
            ), tabItem(
                tabName = "techInfo",
                h2("Technical Information"),
                h3(paste0("chevreulShiny version: ", packageVersion("chevreulShiny"))),
                techInfoui("techInfo")
            )
        )
    )

    ui <- function(request) {
        ui <- dashboardPage(
            header = header, sidebar = sidebar,
            body = body
        )
    }
    server <- function(input, output, session) {

        onBookmark(function(state) {
            state$values$uploadSCEPath <- uploadSCEPath()
        })

        onRestore(function(state) {
            uploadSCEPath(state$values$uploadSCEPath[[1]])
        })

        w <- Waiter$new()

        observe_helpers(
            help_dir = system.file("helpers", package = "chevreulShiny"))
        options(warn = -1)

        con <- reactive({
            dbConnect(
                SQLite(),
                db_path
            )
        })

        projList <- reactivePoll(4000, session, checkFunc = function() {
            if (file.exists(db_path)) {
                dbReadTable(con(), "projects_tbl") |>
                    deframe()
            } else {
                ""
            }
        }, valueFunc = function() {
            dbReadTable(con(), "projects_tbl") |>
                deframe()
        })

        output$projInput <- renderUI({
            selectizeInput("setProject", "Select Project to Load",
                choices = projList(), selected = preset_project,
                multiple = FALSE
            )
        })

        proj_matrices <- reactive({
            create_proj_matrix(projList())
        })

        object <- reactiveVal()
        proj_dir <- reactiveVal()
        uploadSCEPath <- reactiveVal()

        if (!is.null(preset_project)) {
            proj_dir(preset_project)
        }
        organism_type <- reactive({
            input$organism_type
        })
        plot_types <- reactive({
            list_plot_types(object())
        })
        observeEvent(input$loadProject, {
            proj_dir(input$setProject)
        })
        output$appTitle <- renderText({
            req(proj_dir())
            paste0("Loaded Project: ", path_file(proj_dir()))
        })
        dataset_volumes <- reactive({
            message(proj_dir())
            dataset_volumes <- c(
                Home = path(
                    proj_dir(),
                    "output", "singlecellexperiment"
                ), "R Installation" = R.home(),
                getVolumes()()
            )
        })
        observe({
            req(dataset_volumes())
            shinyFileChoose(input, "objectUpload",
                roots = dataset_volumes(), session = session
            )
        })
        observeEvent(input$objectUpload, {
            req(dataset_volumes())

            file <- parseFilePaths(
                dataset_volumes(),
                input$objectUpload
            )
            message(file)
            uploadSCEPath(file$datapath)
        })

        observe({
            req(uploadSCEPath())
            message("uploaded")
            withProgress(
                message = paste0("Uploading Data"),
                value = 0,
                {
                    incProgress(2 / 10)
                    message(uploadSCEPath())
                    updated_sce <- readRDS(uploadSCEPath())
                    object(updated_sce)
                    incProgress(6 / 10)

                    organism <- case_when(
                        str_detect(uploadSCEPath(), "Hs") ~ "human",
                        str_detect(uploadSCEPath(), "Mm") ~ "mouse"
                    )

                    message(uploadSCEPath())
                    message(names(object))
                    incProgress(8 / 10)
                }
            )
        })


        featureType <- reactive({
            "gene"
        })

        observe({
            req(dataset_volumes())
            shinyFileSave(input, "saveSCE",
                # roots = dataset_volumes(),
                roots = c(Home = path(
                    proj_dir(),
                    "output", "singlecellexperiment"
                )),
                session = session, restrictions = system.file(package = "base")
            )
        })
        subSingleCellExperimentPath <- eventReactive(input$saveSCE, {
            req(object())
            savefile <- parseSavePath(
                dataset_volumes(),
                input$saveSCE
            )
            return(savefile$datapath)
        })
        observeEvent(input$saveSCE, {
            req(object())
            req(subSingleCellExperimentPath())

            withProgress(
                message = paste0("Saving Data"),
                value = 0,
                {
                    Sys.sleep(6)
                    incProgress(2 / 10)
                    saveRDS(
                        object(),
                        subSingleCellExperimentPath()
                    )
                    incProgress(10 / 10)
                }
            )
        })

        integrationResults <- integrateProj( "integrateproj",
            proj_matrices, object, proj_dir, con()
        )
        newprojList <- reactive({
            req(integrationResults())
            integration_path <- paste0(integrationResults())
            proj_dir(integration_path)

        names(integration_path) <- path_file(integration_path)

            newprojList <- c(projList(), integration_path)
        })
        observe({
            # print(newprojList())
            updateSelectizeInput(session, "setProject",
                label = "Select input label",
                choices = newprojList(),
                server = TRUE
            )
        })

        observe({
            reformatted_sce <- reformatMetadataDR(
                                             "reformatMetadataDR", object,
                                             featureType)
            object(reformatted_sce())
        })

        reductions <- reactive({
            req(object())
            reducedDimNames(object())
        })

        observe({
            req(object())

            plotDimRed( "plotdimred1", object, plot_types,
                       featureType,
                organism_type = organism_type, reductions
            )
            plotDimRed( "plotdimred2", object, plot_types,
                       featureType,
                organism_type = organism_type, reductions
            )
            plotDimRed( "diffex", object, plot_types, featureType,
                organism_type = organism_type, reductions
            )
            plotDimRed( "subset", object, plot_types, featureType,
                organism_type = organism_type, reductions
            )
            plotDimRed( "markerScatter", object, plot_types,
                       featureType,
                organism_type = organism_type, reductions
            )
        })

        plotReadCount( "plotreadcount1", object, plot_types)
        plotReadCount( "plotreadcount2", object, plot_types)
        plotViolin( "violinPlot", object, featureType,
            organism_type
        )
        plotHeatmap( "heatMap", object, featureType,
            organism_type
        )

        plotCoverage( "coverageplots", object, plot_types, proj_dir,
            organism_type
        )
        plotClustree( "clustreePlot", object)
        tableSelected( "tableselected", object)
        diffex_selected_cells <- tableSelected( "diffex",
            object
        )


        cell_subset <- tableSelected( "subset",
            object
        )
        observeEvent(input$subsetAction, {
            req(cell_subset())
            withCallingHandlers(
                {
                    html("subsetMessages", "")
                    message("Beginning")

                    subset_sce <-
                      object()[, colnames(object()) %in% cell_subset()]
                    object(subset_sce)
                    if (length(unique(object()[["batch"]])) > 1) {
                        message("reintegrating gene expression")
                        reintegrated_sce <- reintegrate_sce(object(),
                            resolution = seq(0.2, 1, by = 0.2),
                            legacy_settings = input$legacySettingsSubset,
                            organism = metadata(object())$experiment$organism
                        )
                        object(reintegrated_sce)
                    } else {
                        subset_sce <- sce_process(
                          object(),
                          resolution = seq(0.2, 1, by = 0.2),
                          legacy_settings = input$legacySettingsSubset)
                        object(subset_sce)
                    }
                    message("Complete!")
                },
                message = function(m) {
                    html(id = "subsetMessages", html = paste0(
                        "Subsetting SingleCellExperiment Object: ",
                        m$message
                    ), add = FALSE)
                }
            )
        })
        observeEvent(input$subsetCsv, {
            req(input$subsetCsv)
            req(input$uploadCsv)
            withCallingHandlers(
                {
                    html("subsetMessages", "")
                    message("Beginning")
                    subset_sce <- subset_by_colData(
                        input$uploadCsv$datapath,
                        object()
                    )
                    object(subset_sce)
                    if (length(unique(object()[["batch"]])) > 1) {
                        message("reintegrating gene expression")
                        reintegrated_sce <- reintegrate_sce(object(),
                            resolution = seq(0.2, 1, by = 0.2),
                            legacy_settings = input$legacySettingsSubset,
                            organism = metadata(object())$experiment$organism
                        )
                        object(reintegrated_sce)
                    } else {
                        subset_sce <- sce_process(object(),
                            resolution = seq(0.2,
                                2,
                                by = 0.2
                            ),
                            legacy_settings = input$legacySettingsSubset
                        )
                        object(subset_sce)
                    }
                    message("Complete!")
                },
                message = function(m) {
                    html(id = "subsetMessages", html = paste0(
                        "Subsetting SingleCellExperiment Object: ",
                        m$message
                    ), add = FALSE)
                }
            )
        })
        observeEvent(input$changeEmbedAction, {
            showModal(modalDialog(
                title = "Recalculating Embedding",
                "This process may take a minute or two!"
            ))
            object <- changeEmbedParams( "changeembed",
                object
            )
            removeModal()
        })
        chevreulMarkers( "findmarkers", object, plot_types,
                   featureType)
        diffex_results <- diffex( "diffex", object, featureType,
            diffex_selected_cells
        )

        observe({
            req(featureType())
            req(object())
            allTranscripts( "alltranscripts1", object, featureType,
                organism_type
            )

            plotDimRed( "alltranscripts2", object, plot_types,
                       featureType,
                organism_type = organism_type, reductions
            )
        })

        observeEvent(input$regressAction, {
            req(object())
            showModal(modalDialog(
                title = "Regressing cycle effects",
                "This process may take a minute or two!"
            ))
            regressed_sce <- regress_cell_cycle(object())
            object(regressed_sce)
            removeModal()
        })

        observe({
            req(uploadSCEPath())
            req(object())

            proj_path <- str_replace(uploadSCEPath(), "output.*", "")

            proj_name <- path_file(proj_path)
            message(proj_name)
        })

        techInfo( "techInfo", object)

        sessionId <- as.integer(runif(1, 1, 100000))
        output$sessionId <- renderText(paste0("Session id: ", sessionId))
        session$onSessionEnded(function() {
            cat(paste0("Ended: ", sessionId))
            observe(dbConnect(con()))
        })
    }

    # onStop(function() dbDisconnect(con))
    shinyApp(ui, server, enableBookmarking = "url")
}

---
title: "chevreulShiny"
author: 
  - name: Kevin Stachelek
    affiliation:
    - University of Southern California   
    email: kevin.stachelek@gmail.com
  - name: Bhavana Bhat
    affiliation:
    - University of Southern California   
    email: bbhat@usc.edu
output: 
  BiocStyle::html_document:
    self_contained: yes
    toc: true
    toc_float: true
    toc_depth: 2
    code_folding: show
date: "`r doc_date()`"
package: "`r pkg_ver('chevreulShiny')`"
vignette: >
  %\VignetteIndexEntry{Preprocessing}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}  
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
    collapse = TRUE,
    comment = "#>",
    dpi = 900,
    out.width = "100%",
    message = FALSE,
    warning = FALSE,
    crop = NULL)

## Related to https://stat.ethz.ch/pipermail/bioc-devel/2020-April/016656.html
```


# Basics

## Install `chevreulShiny`

`R` is an open-source statistical environment which can be easily modified 
to enhance its functionality via packages. `r Biocpkg("chevreulShiny")` is a `R` 
package available via the [Bioconductor](http://bioconductor.org) repository 
for packages. `R` can be installed on any operating system from 
[CRAN](https://cran.r-project.org/) after which you can install 
`r Biocpkg("chevreulShiny")` by using the following commands in your `R` session:

```{r "install", eval = FALSE}
if (!requireNamespace("BiocManager", quietly = TRUE)) {
    install.packages("BiocManager")
}

BiocManager::install("chevreulShiny")

```

## Required knowledge

The `r Biocpkg("chevreulShiny")` package is designed for single-cell RNA sequencing
 data. The functions included within this package are derived from other 
 packages that have implemented the infrastructure needed for RNA-seq data
  processing and analysis. Packages that have been instrumental in the 
  development of `r Biocpkg("chevreulShiny")` include, 
  `Biocpkg("SummarizedExperiment")` and `Biocpkg("scater")`.

## Asking for help

`R` and `Bioconductor` have a steep learning curve so it is critical to 
learn where to ask for help. The 
[Bioconductor support site](https://support.bioconductor.org/) is the main 
resource for getting help: remember to use the `chevreulShiny` tag and check 
[the older posts](https://support.bioconductor.org/tag/chevreulShiny/). 


# Quick start to using `chevreulShiny`

The `chevreulShiny` package contains a shiny app for easy 
visualization and analysis of scRNA data.

`chevreulShiny` uses SingleCellExperiment (SCE) object type 
(from `r Biocpkg("SingleCellExperiment")`) 
to store expression and other metadata from single-cell experiments. 

This package features functions capable of:

* Performing Clustering at a range of resolutions and Dimensional 
reduction of Raw Sequencing Data.
* Visualizing scRNA data using different plotting functions.
* Integration of multiple datasets for consistent analyses. 
* Cell cycle state regression and labeling.

```{r, message=FALSE}

library("chevreulShiny")

# Load the data
data("small_example_dataset")
```

## Shiny app

chevreulShiny includes a shiny app for exploratory scRNA data analysis and 
visualization which can be accessed via

```{r "start", message=FALSE, eval = FALSE}

minimalChevreulApp(small_example_dataset)

```

Note: the SCE object must be pre-processed and integrated (if required) 
prior to running the shiny app.

This package includes a set of Shiny apps for interactive exploration of single cell RNA sequencing (scRNA-seq) datasets processed as
<a href="https://bioconductor.org/packages/devel/bioc/vignettes/SingleCellExperiment/inst/doc/intro.html" target="_blank" rel="noopener noreferrer">SingleCellExperiments</a>

A demo with a developing human retina scRNA-seq dataset from Shayler et al. is available <a href="https://cobrinik-1.saban-chla.usc.edu/shinyproxy/app/chevreul" target="_blank" rel="noopener noreferrer">here</a>

chevreulShiny includes tools for:

-   Interactive data analysis and visualization 

## Installation instructions

chevreulShiny depends on a minimum R version 4.4. The current  
chevreulShiny loads two [Bioconductor](http://bioconductor.org/) dependencies  

1. [`chevreulProcess`](https://github.com/cobriniklab/chevreulProcess)
2. [`chevreulPlot`](https://github.com/cobriniklab/chevreulPlot)

These enable standardized processing, plotting of SingleCellExperiments, respectively.

### Troubleshooting installation

#### Dependency management

When installing an R package like chevreulShiny with many dependencies, conflicts with existing installations can arise. This is a common issue in R package management. Here are some strategies to address this problem:

1. Consider <a href="https://rstudio.github.io/renv/articles/renv.html" target="_blank" rel="noopener noreferrer">renv</a> for dependency management. This tool creates isolated environments for each project, ensuring that package versions don't conflict across different projects.

2. Use the conflicted Package
The <a href="https://conflicted.r-lib.org" target="_blank" rel="noopener noreferrer">conflicted</a> package provides an alternative conflict resolution strategy. It makes every conflict an error, forcing you to choose which function to use

#### Slow internet connection

When installing R packages on slow internet connections, several issues can arise, particularly with larger packages or when using functions like remotes::install_github(). Here are some strategies to address bandwidth-related problems:

Set a longer timeout for downloads:
`options(timeout = 9999999)`

Specify the download method:
`options(download.file.method = "libcurl")`

## Hardware requirements 

Recommended minimum hardware requirements for running chevreulShiny are as follows:
    
- RAM: A minimum of 16 GB RAM is recommended for initial analysis. However, for larger datasets or more complex analyses, 64 GB or more is advisable.
- CPU: Having multiple cores can be beneficial for parallel processing. 
- Storage: Sufficient storage space is necessary, especially for temporary files. The exact amount depends on the size of your datasets
- R Version: Chevruel requires R version 4.4 or greater

It's important to note that these requirements can vary based on the size and complexity of your dataset. As the number of cells increases, so do the hardware requirements. For instance: A dataset with around 8,000 cells can be analyzed with 8 GB of RAM. For larger datasets or more complex analyses, 64-128 GB of RAM can be beneficial.

## Learn More

To learn more about the usage of Bioconductor tools for single-cell RNA-seq analysis. Consult the book <a href="https://bioconductor.org/books/release/OSCA/" target="_blank" rel="noopener noreferrer">Orchestrating Single-Cell Analysis with Bioconductor</a>. The book walks through common workflows for the analysis of single-cell RNA-seq data (scRNA-seq). This book will show you how to make use of cutting-edge Bioconductor tools to process, analyze, visualize, and explore scRNA-seq data

`R` session information.

```{r reproduce3, echo=FALSE}
## Session info
options(width = 120)
sessionInfo()
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wiggleplotr.R
\name{load_bigwigs}
\alias{load_bigwigs}
\title{Load Bigwigs}
\usage{
load_bigwigs(object, bigwig_db = "~/.cache/chevreul/bw-files.db")
}
\arguments{
\item{object}{A object}

\item{bigwig_db}{Sqlite database of bigwig files}
}
\value{
a vector of bigwigs file paths
}
\description{
Load a tibble of bigwig file paths by cell id
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{cc.genes.cyclone}
\alias{cc.genes.cyclone}
\title{Cyclone cell cycle pairs by symbol}
\format{
a list of dataframes with G1, G2, and S gene expression
\describe{
\item{G1}{G1 gene symbols}
\item{G2}{G2 gene symbols}
\item{S}{S gene symbols}
...
}
}
\source{
cyclone
}
\usage{
cc.genes.cyclone
}
\description{
cell cycle genes with paired expression represented by HGNC symbol
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/project_management.R
\name{subset_by_colData}
\alias{subset_by_colData}
\title{Subset by new colData}
\usage{
subset_by_colData(colData_path, object)
}
\arguments{
\item{colData_path}{Path to new colData}

\item{object}{A object}
}
\value{
a SingleCellExperiment object
}
\description{
Subset the object using new colData
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/project_management.R
\name{load_alabaster_from_proj}
\alias{load_alabaster_from_proj}
\title{Load SingleCellExperiment Files from a single project path}
\usage{
load_alabaster_from_proj(proj_dir, ...)
}
\arguments{
\item{proj_dir}{project directory}

\item{...}{extra args passed to load_alabaster_path}
}
\value{
a SingleCellExperiment object
}
\description{
Load SingleCellExperiment Files from a single project path
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/grch38.R
\docType{data}
\name{grch38}
\alias{grch38}
\title{Human annotation data}
\format{
An object of class \code{tbl_df} (inherits from \code{tbl}, \code{data.frame}) with 76062 rows and 9 columns.
}
\source{
\url{http://ensembl.org/homo_sapiens}
}
\usage{
grch38
}
\description{
Human (\emph{Homo sapiens}) annotations based on
genome assembly GRCH38 from Ensembl.
}
\details{
Variables:
\itemize{
\item ensgene
\item entrez
\item symbol
\item chr
\item start
\item end
\item strand
\item biotype
\item description
}
}
\examples{
data("grch38")
head(grch38)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/grch38_tx2gene.R
\docType{data}
\name{grch38_tx2gene}
\alias{grch38_tx2gene}
\title{Human transcripts to genes}
\format{
An object of class \code{tbl_df} (inherits from \code{tbl}, \code{data.frame}) with 277081 rows and 2 columns.
}
\source{
\url{http://ensembl.org/homo_sapiens}
}
\usage{
grch38_tx2gene
}
\description{
Lookup table for converting Human (\emph{Homo sapiens})
Ensembl transcript IDs to gene IDs based on genome assembly
GRCH38 from Ensembl.
}
\details{
Variables:
\itemize{
\item enstxp
\item ensgene
}
}
\examples{
data(grch38_tx2gene)
head(grch38_tx2gene)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{tiny_sce}
\alias{tiny_sce}
\title{Tiny example SingleCellExperiment}
\format{
An SCE with only expression of NRL gene and NRL transripts
}
\source{
chevreuldata::human_gene_transcript_sce()
}
\usage{
tiny_sce
}
\description{
subset to only NRL from chevreuldata::human_gene_transcript_sce()
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{plotly_settings}
\alias{plotly_settings}
\title{Plotly settings}
\usage{
plotly_settings(plotly_plot, width = 600, height = 700)
}
\arguments{
\item{plotly_plot}{A plotly plot}

\item{width}{Default set to '600'}

\item{height}{Default set to '700'}
}
\value{
a plotly plot with settings changed
}
\description{
Change settings of a plotly plot
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/project_management.R
\name{create_proj_matrix}
\alias{create_proj_matrix}
\title{Create a Table of single Cell Projects}
\usage{
create_proj_matrix(proj_list)
}
\arguments{
\item{proj_list}{List of projects}
}
\value{
a tibble of single cell projects
}
\description{
Uses a list of projects to create a matrix of single cell projects
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chevreulShiny-package.R
\docType{package}
\name{chevreulShiny-package}
\alias{chevreulShiny}
\alias{chevreulShiny-package}
\title{chevreulShiny: Tools for managing SingleCellExperiment objects as projects}
\description{
Tools for managing SingleCellExperiment objects as projects. Includes functions for analysis and visualization of single-cell data. Also included is a shiny app for visualization of pre-processed scRNA data. Supported by NIH grants R01CA137124 and R01EY026661 to David Cobrinik.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/whtns/chevreulShiny}
  \item \url{https://whtns.github.io/chevreulShiny/}
  \item Report bugs at \url{https://github.com/cobriniklab/chevreulShiny/issues}
}

}
\author{
\strong{Maintainer}: Kevin Stachelek \email{kevin.stachelek@gmail.com} (\href{https://orcid.org/0000-0003-2085-695X}{ORCID})

Authors:
\itemize{
  \item Bhavana Bhat \email{bhavanabhat29@gmail.com}
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/project_management.R
\name{load_alabaster_path}
\alias{load_alabaster_path}
\title{Read in Gene and Transcript SingleCellExperiment Objects}
\usage{
load_alabaster_path(proj_dir = getwd(), prefix = "unfiltered")
}
\arguments{
\item{proj_dir}{path to project directory}

\item{prefix}{default "unfiltered"}
}
\value{
a SingleCellExperiment object
}
\description{
Read in Gene and Transcript SingleCellExperiment Objects
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wiggleplotr.R
\name{plot_gene_coverage_by_var}
\alias{plot_gene_coverage_by_var}
\title{Plot BigWig Coverage for Genes of Interest by a Given Variable}
\usage{
plot_gene_coverage_by_var(
  genes_of_interest = "NRL",
  cell_metadata,
  bigwig_tbl,
  group_by = "batch",
  values_of_interest = NULL,
  organism = c("human", "mouse"),
  edb = NULL,
  heights = c(3, 1),
  scale_y = "log10",
  reverse_x = FALSE,
  start = NULL,
  end = NULL,
  summarize_transcripts = FALSE,
  ...
)
}
\arguments{
\item{genes_of_interest}{Gene of interest}

\item{cell_metadata}{a dataframe with cell metadata from object}

\item{bigwig_tbl}{a tibble with colnames "name", "bigWig", and "sample_id"
matching the filename, absolute path, and sample name of each cell in the
cell_metadata}

\item{group_by}{Variable to color by}

\item{values_of_interest}{values of interest}

\item{organism}{human (default) or mouse}

\item{edb}{ensembldb object}

\item{heights}{The heights of each row in the grid of plot}

\item{scale_y}{whether to scale coverage}

\item{reverse_x}{whether to reverse x axis}

\item{start}{start coordinates}

\item{end}{end coordinates}

\item{summarize_transcripts}{whether to summarize transcript counts}

\item{...}{extra arguments passed to plotCoverageFromEnsembldb}
}
\value{
a ggplot with coverage faceted by group_by
}
\description{
Plot BigWig coverage for genes of interest colored by a given variable
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{metadata_from_batch}
\alias{metadata_from_batch}
\title{Retrieve Metadata from Batch}
\usage{
metadata_from_batch(
  batch,
  projects_dir = "/dataVolume/storage/single_cell_projects",
  db_path = "single-cell-projects.db"
)
}
\arguments{
\item{batch}{batch}

\item{projects_dir}{path to project dir}

\item{db_path}{path to .db file}
}
\value{
a tibble with cell level metadata from a SingleCellExperiment object
}
\description{
Retrieve Metadata from Batch
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{get_transcripts_from_sce}
\alias{get_transcripts_from_sce}
\title{Get Transcripts in object}
\usage{
get_transcripts_from_sce(object, gene)
}
\arguments{
\item{object}{A SingleCellExperiment object}

\item{gene}{Gene of interest}
}
\value{
transcripts constituting a
gene of interest in a SingleCellExperiment object
}
\description{
Get transcript ids in objects for one or more gene of interest
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{save_sce}
\alias{save_sce}
\title{Save object to \if{html}{\out{<project>}}/output/sce/\if{html}{\out{<feature>}}_sce.rds}
\usage{
save_sce(object, prefix = "unfiltered", proj_dir = getwd())
}
\arguments{
\item{object}{a SingleCellExperiment object}

\item{prefix}{a prefix for saving}

\item{proj_dir}{path to a project directory}
}
\value{
a path to an rds file containing a SingleCellExperiment object
}
\description{
Save object to \if{html}{\out{<project>}}/output/sce/\if{html}{\out{<feature>}}_sce.rds
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/minimal_chevreul_app.R
\name{minimalChevreulApp}
\alias{minimalChevreulApp}
\title{Create a minimal chevreulShiny app using SingleCellExperiment input}
\usage{
minimalChevreulApp(
  single_cell_sce = NULL,
  appTitle = NULL,
  organism_type = "human",
  futureMb = 13000,
  db_name = "single-cell-projects.db"
)
}
\arguments{
\item{single_cell_sce}{a singlecell object}

\item{appTitle}{a title for the app}

\item{organism_type}{human or mouse}

\item{futureMb}{the megabytes available for the future package}

\item{db_name}{a database of bigwig files}
}
\value{
a minimal chevreulShiny app
}
\description{
Create a minimal chevreulShiny app using SingleCellExperiment input
}
\examples{

if (interactive() ) {
  data("tiny_sce")
  minimalChevreulApp(tiny_sce)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chevreul_app.R
\name{chevreulApp}
\alias{chevreulApp}
\title{Create a shiny app for a project on disk}
\usage{
chevreulApp(
  preset_project,
  appTitle = "chevreul",
  organism_type = "human",
  futureMb = 13000,
  db_name = "single-cell-projects.db"
)
}
\arguments{
\item{preset_project}{A preloaded project to start the app with}

\item{appTitle}{A title of the App}

\item{organism_type}{human or mouse}

\item{futureMb}{amount of Mb allocated to future package}

\item{db_name}{sqlite database with list of saved
SingleCellExperiment objects}
}
\value{
a shiny app
}
\description{
Create a shiny app for a project on disk
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{make_chevreul_clean_names}
\alias{make_chevreul_clean_names}
\title{Clean Vector of chevreulShiny Names}
\usage{
make_chevreul_clean_names(myvec)
}
\arguments{
\item{myvec}{A vector of object names}
}
\value{
a clean vector of object names
}
\description{
Cleans names of objects provided in a vector form
}
\examples{

data(small_example_dataset)
make_chevreul_clean_names(colnames(
get_colData(small_example_dataset)))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{human_to_mouse_homologs}
\alias{human_to_mouse_homologs}
\title{Gene Homologs Between Human and Mouse}
\format{
A data frame with 23188 rows and 2 columns
\describe{
\item{HGNC.symbol}{human gene symbols}
\item{MGI.symbol}{mouse gene symbols}
...
}
}
\source{
bioMart
}
\usage{
human_to_mouse_homologs
}
\description{
Homologs drawn from Biomart
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{list_plot_types}
\alias{list_plot_types}
\title{Collate list of variables to be plotted}
\usage{
list_plot_types(object)
}
\arguments{
\item{object}{a SingleCellExperiment object}
}
\value{
plot_types a list of category_vars or continuous_vars
}
\description{
Collate list of variables to be plotted
}
\examples{

data(small_example_dataset)
list_plot_types(small_example_dataset)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{unite_metadata}
\alias{unite_metadata}
\title{Unite metadata}
\usage{
unite_metadata(object, group_bys)
}
\arguments{
\item{object}{A SingleCellExperiment object}

\item{group_bys}{A feature or variable to combine}
}
\value{
a SingleCellExperiment object with Idents formed from concatenation of group_bys
}
\description{
Unite metadata
}
\examples{


data(small_example_dataset)
unite_metadata(small_example_dataset, "Mutation_Status")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{create_project_db}
\alias{create_project_db}
\title{Create a database of chevreulShiny projects}
\usage{
create_project_db(
  cache_location = "~/.cache/chevreul",
  sqlite_db = "single-cell-projects.db",
  verbose = TRUE
)
}
\arguments{
\item{cache_location}{Path to cache "~/.cache/chevreul"}

\item{sqlite_db}{Database to be created}

\item{verbose}{print messages}
}
\value{
a sqlite database with SingleCellExperiment objects
}
\description{
Create a database containing chevreulShiny projects
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{append_to_project_db}
\alias{append_to_project_db}
\title{Update a database of chevreulShiny projects}
\usage{
append_to_project_db(
  new_project_path,
  cache_location = "~/.cache/chevreul",
  sqlite_db = "single-cell-projects.db",
  verbose = TRUE
)
}
\arguments{
\item{new_project_path}{new project path}

\item{cache_location}{Path to cache "~/.cache/chevreul"}

\item{sqlite_db}{sqlite db}

\item{verbose}{print messages}
}
\value{
a sqlite database with SingleCellExperiment objects
}
\description{
Append projects to database
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{make_bigwig_db}
\alias{make_bigwig_db}
\title{Make Bigwig Database}
\usage{
make_bigwig_db(
  new_project = NULL,
  cache_location = "~/.cache/chevreul/",
  sqlite_db = "bw-files.db"
)
}
\arguments{
\item{new_project}{Project directory}

\item{cache_location}{Path to cache "~/.cache/chevreul"}

\item{sqlite_db}{sqlite db containing bw files}
}
\value{
a sqlite database of bigwig files for cells
in a SingleCellExperiment object
}
\description{
Make Bigwig Database
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{small_example_dataset}
\alias{small_example_dataset}
\title{Small example SingleCellExperiment}
\format{
An SCE with 200 cells and 1000 genes
}
\source{
scuttle::mockSCE
}
\usage{
small_example_dataset
}
\description{
created with scuttle::mockSCE
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ensembl_version.R
\docType{data}
\name{ensembl_version}
\alias{ensembl_version}
\title{Ensembl version used for build}
\format{
An object of class \code{character} of length 1.
}
\source{
\url{http://www.ensembl.org/}
}
\usage{
ensembl_version
}
\description{
Ensembl version used for build
}
\examples{
# ensembl_version

}
\keyword{datasets}
